@@ -65,22 +65,21 @@ namespace aria2 {
  * be tested again. Otherwise, it doesn't return anymore mirrors.
  */
 
-AdaptiveURISelector::AdaptiveURISelector
-(const SharedHandle<ServerStatMan>& serverStatMan, RequestGroup* requestGroup)
-  : serverStatMan_(serverStatMan),
-    requestGroup_(requestGroup)
+AdaptiveURISelector::AdaptiveURISelector(
+    std::shared_ptr<ServerStatMan> serverStatMan, RequestGroup* requestGroup)
+    : serverStatMan_(std::move(serverStatMan)), requestGroup_(requestGroup)
 {
   resetCounters();
 }
 
 AdaptiveURISelector::~AdaptiveURISelector() {}
 
-std::string AdaptiveURISelector::select
-(FileEntry* fileEntry,
- const std::vector<std::pair<size_t, std::string> >& usedHosts)
+std::string AdaptiveURISelector::select(
+    FileEntry* fileEntry,
+    const std::vector<std::pair<size_t, std::string>>& usedHosts)
 {
-  A2_LOG_DEBUG(fmt("AdaptiveURISelector: called %d",
-                   requestGroup_->getNumConnection()));
+  A2_LOG_DEBUG(
+      fmt("AdaptiveURISelector: called %d", requestGroup_->getNumConnection()));
   std::deque<std::string>& uris = fileEntry->getRemainingUris();
   if (uris.empty() && requestGroup_->getNumConnection() <= 1) {
     // here we know the download will fail, trying to find previously
@@ -90,31 +89,36 @@ std::string AdaptiveURISelector::select
 
   std::string selected = selectOne(uris);
 
-  if(selected != A2STR::NIL)
-    uris.erase(std::find(uris.begin(), uris.end(), selected));
-
+  if (selected != A2STR::NIL) {
+    uris.erase(std::find(std::begin(uris), std::end(uris), selected));
+  }
   return selected;
 }
 
+namespace {
+constexpr auto MAX_TIMEOUT = 60_s;
+} // namespace
+
 void AdaptiveURISelector::mayRetryWithIncreasedTimeout(FileEntry* fileEntry)
 {
-  if (requestGroup_->getTimeout()*2 >= MAX_TIMEOUT) return;
-  requestGroup_->setTimeout(requestGroup_->getTimeout()*2);
+  if (requestGroup_->getTimeout() * 2 >= MAX_TIMEOUT)
+    return;
+  requestGroup_->setTimeout(requestGroup_->getTimeout() * 2);
 
   std::deque<std::string>& uris = fileEntry->getRemainingUris();
   // looking for retries
   std::deque<URIResult> timeouts;
   fileEntry->extractURIResult(timeouts, error_code::TIME_OUT);
-  std::transform(timeouts.begin(), timeouts.end(), std::back_inserter(uris),
-                 std::mem_fun_ref(&URIResult::getURI));
+  std::transform(std::begin(timeouts), std::end(timeouts),
+                 std::back_inserter(uris), std::mem_fn(&URIResult::getURI));
 
-  if(A2_LOG_DEBUG_ENABLED) {
-    for(std::deque<std::string>::const_iterator i = uris.begin(),
-          eoi = uris.end(); i != eoi; ++i) {
-      A2_LOG_DEBUG(fmt("AdaptiveURISelector: will retry server with increased"
-                       " timeout (%ld s): %s",
-                       static_cast<long int>(requestGroup_->getTimeout()),
-                       (*i).c_str()));
+  if (A2_LOG_DEBUG_ENABLED) {
+    for (const auto& uri : uris) {
+      A2_LOG_DEBUG(
+          fmt("AdaptiveURISelector: will retry server with increased"
+              " timeout (%ld s): %s",
+              static_cast<long int>(requestGroup_->getTimeout().count()),
+              uri.c_str()));
     }
   }
 }
@@ -122,25 +126,27 @@ void AdaptiveURISelector::mayRetryWithIncreasedTimeout(FileEntry* fileEntry)
 std::string AdaptiveURISelector::selectOne(const std::deque<std::string>& uris)
 {
 
-  if(uris.empty()) {
+  if (uris.empty()) {
     return A2STR::NIL;
-  } else {
+  }
+  else {
     const size_t numPieces =
-      requestGroup_->getDownloadContext()->getNumPieces();
+        requestGroup_->getDownloadContext()->getNumPieces();
 
-    bool reservedContext = numPieces > 0 &&
-      static_cast<size_t>(nbConnections_) > std::min
-      (numPieces,
-       static_cast<size_t>(requestGroup_->getNumConcurrentCommand()));
+    bool reservedContext =
+        numPieces > 0 &&
+        static_cast<size_t>(nbConnections_) >
+            std::min(numPieces, static_cast<size_t>(
+                                    requestGroup_->getNumConcurrentCommand()));
     bool selectBest = numPieces == 0 || reservedContext;
 
-    if(numPieces > 0)
+    if (numPieces > 0)
       ++nbConnections_;
 
     /* At least, 3 mirrors must be tested */
-    if(getNbTestedServers(uris) < 3) {
+    if (getNbTestedServers(uris) < 3) {
       std::string notTested = getFirstNotTestedUri(uris);
-      if(notTested != A2STR::NIL) {
+      if (notTested != A2STR::NIL) {
         A2_LOG_DEBUG(fmt("AdaptiveURISelector: choosing the first non tested"
                          " mirror: %s",
                          notTested.c_str()));
@@ -149,24 +155,26 @@ std::string AdaptiveURISelector::selectOne(const std::deque<std::string>& uris)
       }
     }
 
-    if(!selectBest && nbConnections_ > 1 && nbServerToEvaluate_ > 0) {
+    if (!selectBest && nbConnections_ > 1 && nbServerToEvaluate_ > 0) {
       nbServerToEvaluate_--;
       std::string notTested = getFirstNotTestedUri(uris);
-      if(notTested != A2STR::NIL) {
+      if (notTested != A2STR::NIL) {
         /* Here we return the first untested mirror */
         A2_LOG_DEBUG(fmt("AdaptiveURISelector: choosing non tested mirror %s"
                          " for connection #%d",
                          notTested.c_str(), nbConnections_));
         return notTested;
-      } else {
+      }
+      else {
         /* Here we return a mirror which need to be tested again */
         std::string toReTest = getFirstToTestUri(uris);
-        if(toReTest != A2STR::NIL) {
+        if (toReTest != A2STR::NIL) {
           A2_LOG_DEBUG(fmt("AdaptiveURISelector: choosing mirror %s which has"
                            " not been tested recently for connection #%d",
                            toReTest.c_str(), nbConnections_));
           return toReTest;
-        } else {
+        }
+        else {
           return getBestMirror(uris);
         }
       }
@@ -177,28 +185,26 @@ std::string AdaptiveURISelector::selectOne(const std::deque<std::string>& uris)
   }
 }
 
-std::string AdaptiveURISelector::getBestMirror
-(const std::deque<std::string>& uris) const
+std::string
+AdaptiveURISelector::getBestMirror(const std::deque<std::string>& uris) const
 {
   /* Here we return one of the bests mirrors */
   int max = getMaxDownloadSpeed(uris);
-  int min = max-(int)(max*0.25);
+  int min = max - (int)(max * 0.25);
   std::deque<std::string> bests = getUrisBySpeed(uris, min);
 
   if (bests.size() < 2) {
     std::string uri = getMaxDownloadSpeedUri(uris);
     A2_LOG_DEBUG(fmt("AdaptiveURISelector: choosing the best mirror :"
                      " %.2fKB/s %s (other mirrors are at least 25%% slower)",
-                     (float) max/1024,
-                     uri.c_str()));
+                     (float)max / 1024, uri.c_str()));
     return uri;
-  } else {
+  }
+  else {
     std::string uri = selectRandomUri(bests);
     A2_LOG_DEBUG(fmt("AdaptiveURISelector: choosing randomly one of the best"
                      " mirrors (range [%.2fKB/s, %.2fKB/s]): %s",
-                     (float) min/1024,
-                     (float) max/1024,
-                     uri.c_str()));
+                     (float)min / 1024, (float)max / 1024, uri.c_str()));
     return uri;
   }
 }
@@ -206,163 +212,156 @@ std::string AdaptiveURISelector::getBestMirror
 void AdaptiveURISelector::resetCounters()
 {
   nbConnections_ = 1;
-  nbServerToEvaluate_ =
-    requestGroup_->getOption()->getAsInt(PREF_SPLIT) - 1;
+  nbServerToEvaluate_ = requestGroup_->getOption()->getAsInt(PREF_SPLIT) - 1;
 }
 
-void AdaptiveURISelector::tuneDownloadCommand
-(const std::deque<std::string>& uris, DownloadCommand* command)
+void AdaptiveURISelector::tuneDownloadCommand(
+    const std::deque<std::string>& uris, DownloadCommand* command)
 {
   adjustLowestSpeedLimit(uris, command);
 }
 
-void AdaptiveURISelector::adjustLowestSpeedLimit
-(const std::deque<std::string>& uris, DownloadCommand* command) const
+void AdaptiveURISelector::adjustLowestSpeedLimit(
+    const std::deque<std::string>& uris, DownloadCommand* command) const
 {
-  int lowest =
-    requestGroup_->getOption()->getAsInt(PREF_LOWEST_SPEED_LIMIT);
+  int lowest = requestGroup_->getOption()->getAsInt(PREF_LOWEST_SPEED_LIMIT);
   if (lowest > 0) {
-    int low_lowest = 4 * 1024;
+    int low_lowest = 4_k;
     int max = getMaxDownloadSpeed(uris);
     if (max > 0 && lowest > max / 4) {
       A2_LOG_NOTICE(fmt(_("Lowering lowest-speed-limit since known max speed is"
                           " too near (new:%d was:%d max:%d)"),
-                        max / 4,
-                        lowest,
-                        max));
+                        max / 4, lowest, max));
       command->setLowestDownloadSpeedLimit(max / 4);
-    } else if (max == 0 && lowest > low_lowest) {
+    }
+    else if (max == 0 && lowest > low_lowest) {
       A2_LOG_NOTICE(fmt(_("Lowering lowest-speed-limit since we have no clue"
                           " about available speed (now:%d was:%d)"),
-                        low_lowest,
-                        lowest));
+                        low_lowest, lowest));
       command->setLowestDownloadSpeedLimit(low_lowest);
     }
   }
 }
 
 namespace {
-int getUriMaxSpeed(SharedHandle<ServerStat> ss)
+int getUriMaxSpeed(std::shared_ptr<ServerStat> ss)
 {
   return std::max(ss->getSingleConnectionAvgSpeed(),
                   ss->getMultiConnectionAvgSpeed());
 }
 } // namespace
 
-int AdaptiveURISelector::getMaxDownloadSpeed
-(const std::deque<std::string>& uris) const
+int AdaptiveURISelector::getMaxDownloadSpeed(
+    const std::deque<std::string>& uris) const
 {
   std::string uri = getMaxDownloadSpeedUri(uris);
-  if(uri == A2STR::NIL)
+  if (uri == A2STR::NIL)
     return 0;
   return getUriMaxSpeed(getServerStats(uri));
 }
 
-std::string AdaptiveURISelector::getMaxDownloadSpeedUri
-(const std::deque<std::string>& uris) const
+std::string AdaptiveURISelector::getMaxDownloadSpeedUri(
+    const std::deque<std::string>& uris) const
 {
   int max = -1;
   std::string uri = A2STR::NIL;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
-    if(!ss)
+  for (auto& u : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(u);
+    if (!ss)
       continue;
 
-    if((int)ss->getSingleConnectionAvgSpeed() > max) {
+    if ((int)ss->getSingleConnectionAvgSpeed() > max) {
       max = ss->getSingleConnectionAvgSpeed();
-      uri = (*i);
+      uri = u;
     }
-    if((int)ss->getMultiConnectionAvgSpeed() > max) {
+    if ((int)ss->getMultiConnectionAvgSpeed() > max) {
       max = ss->getMultiConnectionAvgSpeed();
-      uri = (*i);
+      uri = u;
     }
   }
   return uri;
 }
 
-std::deque<std::string> AdaptiveURISelector::getUrisBySpeed
-(const std::deque<std::string>& uris, int min) const
+std::deque<std::string>
+AdaptiveURISelector::getUrisBySpeed(const std::deque<std::string>& uris,
+                                    int min) const
 {
   std::deque<std::string> bests;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
-    if(!ss)
+  for (auto& uri : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(uri);
+    if (!ss)
       continue;
-    if(ss->getSingleConnectionAvgSpeed() > min ||
-       ss->getMultiConnectionAvgSpeed() > min) {
-      bests.push_back(*i);
+    if (ss->getSingleConnectionAvgSpeed() > min ||
+        ss->getMultiConnectionAvgSpeed() > min) {
+      bests.push_back(uri);
     }
   }
   return bests;
 }
 
-std::string AdaptiveURISelector::selectRandomUri
-(const std::deque<std::string>& uris) const
+std::string
+AdaptiveURISelector::selectRandomUri(const std::deque<std::string>& uris) const
 {
   int pos = SimpleRandomizer::getInstance()->getRandomNumber(uris.size());
-  std::deque<std::string>::const_iterator i = uris.begin();
-  i = i+pos;
+  auto i = std::begin(uris);
+  i = i + pos;
   return *i;
 }
 
-std::string AdaptiveURISelector::getFirstNotTestedUri
-(const std::deque<std::string>& uris) const
+std::string AdaptiveURISelector::getFirstNotTestedUri(
+    const std::deque<std::string>& uris) const
 {
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
-    if(!ss)
-      return *i;
+  for (const auto& i : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(i);
+    if (!ss)
+      return i;
   }
   return A2STR::NIL;
 }
 
-std::string AdaptiveURISelector::getFirstToTestUri
-(const std::deque<std::string>& uris) const
+std::string AdaptiveURISelector::getFirstToTestUri(
+    const std::deque<std::string>& uris) const
 {
   int counter;
   int power;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
-    if(!ss)
+  for (const auto& u : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(u);
+    if (!ss)
       continue;
     counter = ss->getCounter();
-    if(counter > 8)
+    if (counter > 8)
       continue;
     power = (int)pow(2.0, (float)counter);
     /* We test the mirror another time if it has not been
      * tested since 2^counter days */
-    if(ss->getLastUpdated().difference() > power*24*60*60) {
-      return *i;
+    if (ss->getLastUpdated().difference() > std::chrono::hours(power * 24)) {
+      return u;
     }
   }
   return A2STR::NIL;
 }
 
-SharedHandle<ServerStat> AdaptiveURISelector::getServerStats
-(const std::string& uri) const
+std::shared_ptr<ServerStat>
+AdaptiveURISelector::getServerStats(const std::string& uri) const
 {
   uri_split_result us;
-  if(uri_split(&us, uri.c_str()) == 0) {
+  if (uri_split(&us, uri.c_str()) == 0) {
     std::string host = uri::getFieldString(us, USR_HOST, uri.c_str());
     std::string protocol = uri::getFieldString(us, USR_SCHEME, uri.c_str());
     return serverStatMan_->find(host, protocol);
-  } else {
-    return SharedHandle<ServerStat>();
+  }
+  else {
+    return nullptr;
   }
 }
 
-int AdaptiveURISelector::getNbTestedServers
-(const std::deque<std::string>& uris) const
+int AdaptiveURISelector::getNbTestedServers(
+    const std::deque<std::string>& uris) const
 {
   int counter = 0;
-  for(std::deque<std::string>::const_iterator i = uris.begin(),
-        eoi = uris.end(); i != eoi; ++i) {
-    SharedHandle<ServerStat> ss = getServerStats(*i);
-    if(!ss)
+  for (const auto& u : uris) {
+    std::shared_ptr<ServerStat> ss = getServerStats(u);
+    if (!ss)
       ++counter;
   }
   return uris.size() - counter;
