@@ -49,26 +49,26 @@
 #include "DlAbortEx.h"
 #include "fmt.h"
 #include "FileEntry.h"
+#include "SimpleRandomizer.h"
 
 namespace aria2 {
 
-BtDependency::BtDependency
-(RequestGroup* dependant,
- const SharedHandle<RequestGroup>& dependee)
-  : dependant_(dependant),
-    dependee_(dependee)
-{}
+BtDependency::BtDependency(RequestGroup* dependant,
+                           const std::shared_ptr<RequestGroup>& dependee)
+    : dependant_(dependant), dependee_(dependee)
+{
+}
 
 BtDependency::~BtDependency() {}
 
 namespace {
-void copyValues(const SharedHandle<FileEntry>& d,
-                const SharedHandle<FileEntry>& s)
+void copyValues(const std::shared_ptr<FileEntry>& d,
+                const std::shared_ptr<FileEntry>& s)
 {
   d->setRequested(true);
   d->setPath(s->getPath());
-  d->addUris(s->getRemainingUris().begin(),
-             s->getRemainingUris().end());
+  d->addUris(std::begin(s->getRemainingUris()),
+             std::end(s->getRemainingUris()));
   d->setMaxConnectionPerServer(s->getMaxConnectionPerServer());
   d->setUniqueProtocol(s->isUniqueProtocol());
 }
@@ -76,9 +76,8 @@ void copyValues(const SharedHandle<FileEntry>& d,
 
 namespace {
 struct EntryCmp {
-  bool operator()
-  (const SharedHandle<FileEntry>& lhs,
-   const SharedHandle<FileEntry>& rhs) const
+  bool operator()(const std::shared_ptr<FileEntry>& lhs,
+                  const std::shared_ptr<FileEntry>& rhs) const
   {
     return lhs->getOriginalName() < rhs->getOriginalName();
   }
@@ -87,90 +86,96 @@ struct EntryCmp {
 
 bool BtDependency::resolve()
 {
-  if(!dependee_) {
+  if (!dependee_) {
     return true;
   }
-  if(dependee_->getNumCommand() == 0 && dependee_->downloadFinished()) {
-    SharedHandle<RequestGroup> dependee = dependee_;
+  if (dependee_->getNumCommand() == 0 && dependee_->downloadFinished()) {
+    std::shared_ptr<RequestGroup> dependee = dependee_;
     // cut reference here
     dependee_.reset();
-    SharedHandle<DownloadContext> context(new DownloadContext());
+    auto context = std::make_shared<DownloadContext>();
     try {
-      SharedHandle<DiskAdaptor> diskAdaptor =
-        dependee->getPieceStorage()->getDiskAdaptor();
+      std::shared_ptr<DiskAdaptor> diskAdaptor =
+          dependee->getPieceStorage()->getDiskAdaptor();
       diskAdaptor->openExistingFile();
       std::string content = util::toString(diskAdaptor);
-      if(dependee->getDownloadContext()->hasAttribute(bittorrent::BITTORRENT)) {
-        SharedHandle<TorrentAttribute> attrs =
-          bittorrent::getTorrentAttrs(dependee->getDownloadContext());
-        bittorrent::loadFromMemory
-          (bittorrent::metadata2Torrent(content, attrs), context,
-           dependant_->getOption(), "default");
+      if (dependee->getDownloadContext()->hasAttribute(CTX_ATTR_BT)) {
+        auto attrs =
+            bittorrent::getTorrentAttrs(dependee->getDownloadContext());
+        bittorrent::loadFromMemory(bittorrent::metadata2Torrent(content, attrs),
+                                   context, dependant_->getOption(), "default");
         // We don't call bittorrent::adjustAnnounceUri() because it
         // has already been called with attrs.
-      } else {
-        bittorrent::loadFromMemory
-          (content, context, dependant_->getOption(),
-           File(dependee->getFirstFilePath()).getBasename());
+      }
+      else {
+        bittorrent::loadFromMemory(
+            content, context, dependant_->getOption(),
+            File(dependee->getFirstFilePath()).getBasename());
         bittorrent::adjustAnnounceUri(bittorrent::getTorrentAttrs(context),
                                       dependant_->getOption());
       }
-      const std::vector<SharedHandle<FileEntry> >& fileEntries =
-        context->getFileEntries();
-      const std::vector<SharedHandle<FileEntry> >& dependantFileEntries =
-        dependant_->getDownloadContext()->getFileEntries();
+      const std::vector<std::shared_ptr<FileEntry>>& fileEntries =
+          context->getFileEntries();
+      for (auto& fe : fileEntries) {
+        auto& uri = fe->getRemainingUris();
+        std::shuffle(std::begin(uri), std::end(uri),
+                     *SimpleRandomizer::getInstance());
+      }
+      const std::vector<std::shared_ptr<FileEntry>>& dependantFileEntries =
+          dependant_->getDownloadContext()->getFileEntries();
       // If dependant's FileEntry::getOriginalName() is empty, we
       // assume that torrent is single file. In Metalink3, this is
       // always assumed.
-      if(fileEntries.size() == 1 && dependantFileEntries.size() == 1 &&
-         dependantFileEntries[0]->getOriginalName().empty()) {
+      if (fileEntries.size() == 1 && dependantFileEntries.size() == 1 &&
+          dependantFileEntries[0]->getOriginalName().empty()) {
+        // TODO this may be dead code
         copyValues(fileEntries[0], dependantFileEntries[0]);
-      } else {
-        std::vector<SharedHandle<FileEntry> > destFiles;
+      }
+      else {
+        std::vector<std::shared_ptr<FileEntry>> destFiles;
         destFiles.reserve(fileEntries.size());
-        for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-              fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
-          (*i)->setRequested(false);
-          destFiles.push_back(*i);
+        for (auto& e : fileEntries) {
+          e->setRequested(false);
+          destFiles.push_back(e);
         }
-        std::sort(destFiles.begin(), destFiles.end(), EntryCmp());
+        std::sort(std::begin(destFiles), std::end(destFiles), EntryCmp());
         // Copy file path in dependant_'s FileEntries to newly created
         // context's FileEntries to endorse the path structure of
         // dependant_.  URIs and singleHostMultiConnection are also copied.
-        for(std::vector<SharedHandle<FileEntry> >::const_iterator s =
-              dependantFileEntries.begin(), eoi = dependantFileEntries.end();
-            s != eoi; ++s){
-          std::vector<SharedHandle<FileEntry> >::const_iterator d =
-            std::lower_bound(destFiles.begin(), destFiles.end(), *s,
-                             EntryCmp());
-          if(d == destFiles.end() ||
-             (*d)->getOriginalName() != (*s)->getOriginalName()) {
-            throw DL_ABORT_EX
-              (fmt("No entry %s in torrent file",
-                   (*s)->getOriginalName().c_str()));
-          } else {
-            copyValues(*d, *s);
+        for (const auto& e : dependantFileEntries) {
+          const auto d = std::lower_bound(std::begin(destFiles),
+                                          std::end(destFiles), e, EntryCmp());
+          if (d == std::end(destFiles) ||
+              (*d)->getOriginalName() != e->getOriginalName()) {
+            throw DL_ABORT_EX(fmt("No entry %s in torrent file",
+                                  e->getOriginalName().c_str()));
+          }
+          else {
+            copyValues(*d, e);
           }
         }
       }
-    } catch(RecoverableException& e) {
+    }
+    catch (RecoverableException& e) {
       A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
-      A2_LOG_INFO(fmt("BtDependency for GID#%" PRId64 " failed. Go without Bt.",
-                      dependant_->getGID()));
+      A2_LOG_INFO(fmt("BtDependency for GID#%s failed. Go without Bt.",
+                      GroupId::toHex(dependant_->getGID()).c_str()));
       return true;
     }
-    A2_LOG_INFO(fmt("Dependency resolved for GID#%" PRId64 "",
-                    dependant_->getGID()));
+    A2_LOG_INFO(fmt("Dependency resolved for GID#%s",
+                    GroupId::toHex(dependant_->getGID()).c_str()));
     dependant_->setDownloadContext(context);
     return true;
-  } else if(dependee_->getNumCommand() == 0) {
+  }
+  else if (dependee_->getNumCommand() == 0) {
     // dependee_'s download failed.
     // cut reference here
     dependee_.reset();
-    A2_LOG_INFO(fmt("BtDependency for GID#%" PRId64 " failed. Go without Bt.",
-                    dependant_->getGID()));
+    A2_LOG_INFO(fmt("BtDependency for GID#%s failed. Go without Bt.",
+                    GroupId::toHex(dependant_->getGID()).c_str()));
     return true;
-  } else {
+  }
+  else {
     return false;
   }
 }
