@@ -46,24 +46,28 @@
 namespace aria2 {
 
 BtLeecherStateChoke::BtLeecherStateChoke()
-  : round_(0),
-    lastRound_(0)
-{}
+    : round_(0), lastRound_(Timer::zero())
+{
+}
 
 BtLeecherStateChoke::~BtLeecherStateChoke() {}
 
-BtLeecherStateChoke::PeerEntry::PeerEntry(const SharedHandle<Peer>& peer):
-  peer_(peer), downloadSpeed_(peer->calculateDownloadSpeed()),
-  // peer must be interested to us and sent block in the last 30 seconds
-  regularUnchoker_
-  (peer->peerInterested() &&
-   peer->getLastDownloadUpdate().difference(global::wallclock()) < 30) {}
+BtLeecherStateChoke::PeerEntry::PeerEntry(const std::shared_ptr<Peer>& peer)
+    : peer_(peer),
+      downloadSpeed_(peer->calculateDownloadSpeed()),
+      // peer must be interested to us and sent block in the last 30 seconds
+      regularUnchoker_(
+          peer->peerInterested() &&
+          peer->getLastDownloadUpdate().difference(global::wallclock()) < 30_s)
+{
+}
 
 BtLeecherStateChoke::PeerEntry::PeerEntry(const PeerEntry& c)
-  : peer_(c.peer_),
-    downloadSpeed_(c.downloadSpeed_),
-    regularUnchoker_(c.regularUnchoker_)
-{}
+    : peer_(c.peer_),
+      downloadSpeed_(c.downloadSpeed_),
+      regularUnchoker_(c.regularUnchoker_)
+{
+}
 
 void BtLeecherStateChoke::PeerEntry::swap(PeerEntry& c)
 {
@@ -73,10 +77,10 @@ void BtLeecherStateChoke::PeerEntry::swap(PeerEntry& c)
   swap(regularUnchoker_, c.regularUnchoker_);
 }
 
-BtLeecherStateChoke::PeerEntry& BtLeecherStateChoke::PeerEntry::operator=
-(const PeerEntry& c)
+BtLeecherStateChoke::PeerEntry& BtLeecherStateChoke::PeerEntry::
+operator=(const PeerEntry& c)
 {
-  if(this != &c) {
+  if (this != &c) {
     peer_ = c.peer_;
     downloadSpeed_ = c.downloadSpeed_;
     regularUnchoker_ = c.regularUnchoker_;
@@ -86,12 +90,12 @@ BtLeecherStateChoke::PeerEntry& BtLeecherStateChoke::PeerEntry::operator=
 
 BtLeecherStateChoke::PeerEntry::~PeerEntry() {}
 
-const SharedHandle<Peer>& BtLeecherStateChoke::PeerEntry::getPeer() const
+const std::shared_ptr<Peer>& BtLeecherStateChoke::PeerEntry::getPeer() const
 {
   return peer_;
 }
 
-unsigned int BtLeecherStateChoke::PeerEntry::getDownloadSpeed() const
+int BtLeecherStateChoke::PeerEntry::getDownloadSpeed() const
 {
   return downloadSpeed_;
 }
@@ -121,121 +125,124 @@ void BtLeecherStateChoke::PeerEntry::disableOptUnchoking()
   peer_->optUnchoking(false);
 }
 
-bool BtLeecherStateChoke::PeerEntry::isSnubbing() const
-{
-  return peer_->snubbing();
-}
-
 bool BtLeecherStateChoke::PeerEntry::operator<(const PeerEntry& peerEntry) const
 {
   return downloadSpeed_ > peerEntry.downloadSpeed_;
 }
 
-void swap
-(BtLeecherStateChoke::PeerEntry& a,
- BtLeecherStateChoke::PeerEntry& b)
+void swap(BtLeecherStateChoke::PeerEntry& a, BtLeecherStateChoke::PeerEntry& b)
 {
   a.swap(b);
 }
 
-bool BtLeecherStateChoke::PeerFilter::operator()
-  (const PeerEntry& peerEntry) const
+bool BtLeecherStateChoke::PeerFilter::
+operator()(const PeerEntry& peerEntry) const
 {
   return peerEntry.getPeer()->amChoking() == amChoking_ &&
-    peerEntry.getPeer()->peerInterested() == peerInterested_;
+         peerEntry.getPeer()->peerInterested() == peerInterested_;
 }
 
-void BtLeecherStateChoke::plannedOptimisticUnchoke
-(std::vector<PeerEntry>& peerEntries)
+void BtLeecherStateChoke::plannedOptimisticUnchoke(
+    std::vector<PeerEntry>& peerEntries)
 {
-  std::for_each(peerEntries.begin(), peerEntries.end(),
-                std::mem_fun_ref(&PeerEntry::disableOptUnchoking));
-  
-  std::vector<PeerEntry>::iterator i =
-    std::partition(peerEntries.begin(), peerEntries.end(),
-                   PeerFilter(true, true));
-  if(i != peerEntries.begin()) {
-    std::random_shuffle(peerEntries.begin(), i,
-                        *(SimpleRandomizer::getInstance().get()));
-    (*peerEntries.begin()).enableOptUnchoking();
-    A2_LOG_INFO(fmt("POU: %s",
-                    (*peerEntries.begin()).getPeer()->getIPAddress().c_str()));
+  std::for_each(std::begin(peerEntries), std::end(peerEntries),
+                std::mem_fn(&PeerEntry::disableOptUnchoking));
+
+  auto i = std::partition(std::begin(peerEntries), std::end(peerEntries),
+                          PeerFilter(true, true));
+  if (i != std::begin(peerEntries)) {
+    std::shuffle(std::begin(peerEntries), i, *SimpleRandomizer::getInstance());
+
+    auto& ent = *std::begin(peerEntries);
+    auto& peer = ent.getPeer();
+
+    ent.enableOptUnchoking();
+
+    A2_LOG_INFO(
+        fmt("POU: %s:%u", peer->getIPAddress().c_str(), peer->getPort()));
   }
 }
 
 void BtLeecherStateChoke::regularUnchoke(std::vector<PeerEntry>& peerEntries)
 {
-  std::vector<PeerEntry>::iterator rest =
-    std::partition(peerEntries.begin(), peerEntries.end(),
-                   std::mem_fun_ref(&PeerEntry::isRegularUnchoker));
-  
-  std::sort(peerEntries.begin(), rest);
+  auto rest = std::partition(std::begin(peerEntries), std::end(peerEntries),
+                             std::mem_fn(&PeerEntry::isRegularUnchoker));
+
+  std::sort(std::begin(peerEntries), rest);
+  std::shuffle(rest, std::end(peerEntries), *SimpleRandomizer::getInstance());
 
   // the number of regular unchokers
   int count = 3;
 
   bool fastOptUnchoker = false;
-  std::vector<PeerEntry>::iterator peerIter = peerEntries.begin();
-  for(;peerIter != rest && count; ++peerIter, --count) {
-    (*peerIter).disableChokingRequired();
-    A2_LOG_INFO(fmt("RU: %s, dlspd=%u",
-                    (*peerIter).getPeer()->getIPAddress().c_str(),
-                    (*peerIter).getDownloadSpeed()));
-    if((*peerIter).getPeer()->optUnchoking()) {
+  auto peerIter = std::begin(peerEntries);
+  for (; peerIter != std::end(peerEntries) && count; ++peerIter, --count) {
+    auto& peer = peerIter->getPeer();
+
+    if (!peer->peerInterested()) {
+      continue;
+    }
+
+    peerIter->disableChokingRequired();
+
+    A2_LOG_INFO(fmt("RU: %s:%u, dlspd=%d", peer->getIPAddress().c_str(),
+                    peer->getPort(), (*peerIter).getDownloadSpeed()));
+
+    if (peer->optUnchoking()) {
       fastOptUnchoker = true;
-      (*peerIter).disableOptUnchoking();
+      peerIter->disableOptUnchoking();
     }
   }
-  if(fastOptUnchoker) {
-    std::random_shuffle(peerIter, peerEntries.end(),
-                        *(SimpleRandomizer::getInstance().get()));
-    for(std::vector<PeerEntry>::iterator i = peerIter,
-          eoi = peerEntries.end(); i != eoi; ++i) {
-      if((*i).getPeer()->peerInterested()) {
-        (*i).enableOptUnchoking();
-        A2_LOG_INFO(fmt("OU: %s", (*i).getPeer()->getIPAddress().c_str()));
-        break;
-      } else {
-        (*i).disableChokingRequired();
-        A2_LOG_INFO(fmt("OU: %s", (*i).getPeer()->getIPAddress().c_str()));
+  if (fastOptUnchoker) {
+    for (auto& p : peerEntries) {
+      if (!p.getPeer()->peerInterested()) {
+        continue;
       }
+
+      p.enableOptUnchoking();
+
+      auto& peer = p.getPeer();
+
+      A2_LOG_INFO(
+          fmt("OU: %s:%u", peer->getIPAddress().c_str(), peer->getPort()));
+
+      break;
     }
   }
 }
 
-void
-BtLeecherStateChoke::executeChoke
-(const std::vector<SharedHandle<Peer> >& peerSet)
+void BtLeecherStateChoke::executeChoke(const PeerSet& peerSet)
 {
   A2_LOG_INFO(fmt("Leecher state, %d choke round started", round_));
   lastRound_ = global::wallclock();
 
   std::vector<PeerEntry> peerEntries;
-  std::transform(peerSet.begin(), peerSet.end(),
-                 std::back_inserter(peerEntries),
-                 BtLeecherStateChokeGenPeerEntry());
+  for (const auto& p : peerSet) {
+    if (!p->isActive()) {
+      continue;
+    }
 
-  peerEntries.erase(std::remove_if(peerEntries.begin(), peerEntries.end(),
-                                   std::mem_fun_ref(&PeerEntry::isSnubbing)),
-                    peerEntries.end());
-              
-  std::for_each(peerEntries.begin(), peerEntries.end(),
-                std::mem_fun_ref(&PeerEntry::enableChokingRequired));
+    p->chokingRequired(true);
+
+    if (p->snubbing()) {
+      p->optUnchoking(false);
+      continue;
+    }
+
+    peerEntries.push_back(PeerEntry(p));
+  }
 
   // planned optimistic unchoke
-  if(round_ == 0) {
+  if (round_ == 0) {
     plannedOptimisticUnchoke(peerEntries);
   }
   regularUnchoke(peerEntries);
 
-  if(++round_ == 3) {
+  if (++round_ == 3) {
     round_ = 0;
   }
 }
 
-const Timer& BtLeecherStateChoke::getLastRound() const
-{
-  return lastRound_;
-}
+const Timer& BtLeecherStateChoke::getLastRound() const { return lastRound_; }
 
 } // namespace aria2
