@@ -35,41 +35,37 @@
 #include "BtNotInterestedMessage.h"
 #include "Peer.h"
 #include "PeerStorage.h"
+#include "SocketBuffer.h"
 
 namespace aria2 {
 
-const std::string BtNotInterestedMessage::NAME("not interested");
+const char BtNotInterestedMessage::NAME[] = "not interested";
 
-BtNotInterestedMessage::BtNotInterestedMessage():ZeroBtMessage(ID, NAME) {}
+BtNotInterestedMessage::BtNotInterestedMessage()
+    : ZeroBtMessage(ID, NAME), peerStorage_(nullptr)
+{
+}
 
-SharedHandle<BtNotInterestedMessage> BtNotInterestedMessage::create
-(const unsigned char* data, size_t dataLength)
+BtNotInterestedMessage::~BtNotInterestedMessage() {}
+
+std::unique_ptr<BtNotInterestedMessage>
+BtNotInterestedMessage::create(const unsigned char* data, size_t dataLength)
 {
   return ZeroBtMessage::create<BtNotInterestedMessage>(data, dataLength);
 }
 
 void BtNotInterestedMessage::doReceivedAction()
 {
-  if(isMetadataGetMode()) {
+  if (isMetadataGetMode()) {
     return;
   }
   getPeer()->peerInterested(false);
-  if(!getPeer()->amChoking()) {
+  if (!getPeer()->amChoking()) {
     peerStorage_->executeChoke();
   }
 }
 
-bool BtNotInterestedMessage::sendPredicate() const
-{
-  return getPeer()->amInterested();
-}
-
-void BtNotInterestedMessage::onSendComplete() {
-  getPeer()->amInterested(false);
-}
-
-void BtNotInterestedMessage::setPeerStorage
-(const SharedHandle<PeerStorage>& peerStorage)
+void BtNotInterestedMessage::setPeerStorage(PeerStorage* peerStorage)
 {
   peerStorage_ = peerStorage;
 }
