@@ -44,42 +44,48 @@
 
 namespace aria2 {
 
-ChecksumCheckIntegrityEntry::ChecksumCheckIntegrityEntry(RequestGroup* requestGroup, Command* nextCommand):
-  CheckIntegrityEntry(requestGroup, nextCommand),
-  redownload_(false) {}
+ChecksumCheckIntegrityEntry::ChecksumCheckIntegrityEntry(
+    RequestGroup* requestGroup, std::unique_ptr<Command> nextCommand)
+    : CheckIntegrityEntry{requestGroup, std::move(nextCommand)},
+      redownload_{false}
+{
+}
 
 ChecksumCheckIntegrityEntry::~ChecksumCheckIntegrityEntry() {}
 
 bool ChecksumCheckIntegrityEntry::isValidationReady()
 {
-  const SharedHandle<DownloadContext>& dctx =
-    getRequestGroup()->getDownloadContext();
+  const std::shared_ptr<DownloadContext>& dctx =
+      getRequestGroup()->getDownloadContext();
   return dctx->isChecksumVerificationAvailable();
 }
 
 void ChecksumCheckIntegrityEntry::initValidator()
 {
-  SharedHandle<IteratableChecksumValidator> validator
-    (new IteratableChecksumValidator(getRequestGroup()->getDownloadContext(),
-                                     getRequestGroup()->getPieceStorage()));
+  auto validator = make_unique<IteratableChecksumValidator>(
+      getRequestGroup()->getDownloadContext(),
+      getRequestGroup()->getPieceStorage());
   validator->init();
-  setValidator(validator);
+  setValidator(std::move(validator));
 }
 
-void
-ChecksumCheckIntegrityEntry::onDownloadFinished
-(std::vector<Command*>& commands, DownloadEngine* e)
-{}
-
-void
-ChecksumCheckIntegrityEntry::onDownloadIncomplete
-(std::vector<Command*>& commands, DownloadEngine* e)
+void ChecksumCheckIntegrityEntry::onDownloadFinished(
+    std::vector<std::unique_ptr<Command>>& commands, DownloadEngine* e)
 {
-  if(redownload_) {
-    SharedHandle<FileAllocationEntry> entry
-      (new StreamFileAllocationEntry(getRequestGroup(), popNextCommand()));
-    proceedFileAllocation(commands, entry, e);
+}
+
+void ChecksumCheckIntegrityEntry::onDownloadIncomplete(
+    std::vector<std::unique_ptr<Command>>& commands, DownloadEngine* e)
+{
+  if (redownload_) {
+    proceedFileAllocation(commands, make_unique<StreamFileAllocationEntry>(
+                                        getRequestGroup(), popNextCommand()),
+                          e);
+    return;
   }
+
+  // If we don't redownload, set error code to indicate checksum error
+  getRequestGroup()->setLastErrorCode(error_code::CHECKSUM_ERROR);
 }
 
 } // namespace aria2
