@@ -34,8 +34,6 @@
 /* copyright --> */
 #include "ContentTypeRequestGroupCriteria.h"
 
-#include <algorithm>
-
 #include "RequestGroup.h"
 #include "util.h"
 #include "FileEntry.h"
@@ -43,33 +41,34 @@
 
 namespace aria2 {
 
-template<typename InputIterator>
-static bool tailMatch
-(InputIterator first, InputIterator last, const std::string& target)
+ContentTypeRequestGroupCriteria::ContentTypeRequestGroupCriteria(
+    const char** contentTypes, const char** extensions)
+    : contentTypes_(contentTypes), extensions_(extensions)
 {
-  for(; first != last; ++first) {
-    if(util::endsWith(target, *first)) {
+}
+
+ContentTypeRequestGroupCriteria::~ContentTypeRequestGroupCriteria() {}
+
+bool ContentTypeRequestGroupCriteria::match(
+    const RequestGroup* requestGroup) const
+{
+  if (requestGroup->getDownloadContext()->getFileEntries().size() != 1) {
+    return false;
+  }
+  for (size_t i = 0; extensions_[i]; ++i) {
+    if (util::iendsWith(requestGroup->getFirstFilePath(), extensions_[i])) {
+      return true;
+    }
+  }
+  for (size_t i = 0; contentTypes_[i]; ++i) {
+    if (util::strieq(requestGroup->getDownloadContext()
+                         ->getFirstFileEntry()
+                         ->getContentType(),
+                     contentTypes_[i])) {
       return true;
     }
   }
   return false;
 }
 
-bool ContentTypeRequestGroupCriteria::match
-(const RequestGroup* requestGroup) const
-{
-  if(requestGroup->getDownloadContext()->getFileEntries().size() != 1) {
-    return false;
-  }
-  if(tailMatch(extensions_.begin(), extensions_.end(),
-               requestGroup->getFirstFilePath())) {
-    return true;
-  } else {
-    return
-      std::find(contentTypes_.begin(), contentTypes_.end(),
-                requestGroup->getDownloadContext()->getFirstFileEntry()->
-                getContentType()) != contentTypes_.end();
-  }
-}
-
 } // namespace aria2
