@@ -43,7 +43,7 @@ namespace aria2 {
  * Interface for writing to a binary stream of bytes.
  *
  */
-class DiskWriter:public BinaryStream {
+class DiskWriter : public BinaryStream {
 public:
   DiskWriter() {}
 
@@ -51,9 +51,9 @@ public:
   /**
    * Opens file. If the file exists, then it is truncated to 0 length.
    */
-  virtual void initAndOpenFile(off_t totalLength = 0)  = 0;
-  
-  virtual void openFile(off_t totalLength = 0) = 0;
+  virtual void initAndOpenFile(int64_t totalLength = 0) = 0;
+
+  virtual void openFile(int64_t totalLength = 0) = 0;
 
   /**
    * Closes this output stream.
@@ -65,10 +65,10 @@ public:
    * Opens a file.  If the file doesnot exists, an exception may be
    * thrown.
    */
-  virtual void openExistingFile(off_t totalLength = 0) = 0;
+  virtual void openExistingFile(int64_t totalLength = 0) = 0;
 
   // Returns file length
-  virtual off_t size() = 0;
+  virtual int64_t size() = 0;
 
   // Enables read-only mode. After this call, openExistingFile() opens
   // file in read-only mode. This is an optional functionality. The
@@ -79,9 +79,13 @@ public:
   // opens file in read/write mode. This is an optional
   // functionality. The default implementation is do noting.
   virtual void disableReadOnly() {}
-};
 
-typedef SharedHandle<DiskWriter> DiskWriterHandle;
+  // Enables mmap.
+  virtual void enableMmap() {}
+
+  // Drops cache in range [offset, offset + len)
+  virtual void dropCache(int64_t len, int64_t offset) {}
+};
 
 } // namespace aria2
 
