@@ -39,16 +39,16 @@
 
 namespace aria2 {
 
-class HaveEraseCommand : public TimeBasedCommand
-{
+class HaveEraseCommand : public TimeBasedCommand {
 public:
-  HaveEraseCommand(cuid_t cuid, DownloadEngine* e, time_t interval);
+  HaveEraseCommand(cuid_t cuid, DownloadEngine* e,
+                   std::chrono::seconds interval);
 
   virtual ~HaveEraseCommand();
 
-  virtual void preProcess();
+  virtual void preProcess() CXX11_OVERRIDE;
 
-  virtual void process();
+  virtual void process() CXX11_OVERRIDE;
 };
 
 } // namespace aria2
