@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -33,78 +33,77 @@
  */
 /* copyright --> */
 #include "IteratableChecksumValidator.h"
-#include "Util.h"
+
+#include <array>
+#include <cstdlib>
+
+#include "util.h"
 #include "message.h"
-#include "SingleFileDownloadContext.h"
 #include "PieceStorage.h"
-#include "messageDigest.h"
-#include "LogFactory.h"
-#include "Logger.h"
+#include "MessageDigest.h"
 #include "DiskAdaptor.h"
+#include "FileEntry.h"
 #include "BitfieldMan.h"
+#include "DownloadContext.h"
+#include "LogFactory.h"
+#include "fmt.h"
 
-#define BUFSIZE (256*1024)
-#define ALIGNMENT 512
+namespace aria2 {
 
-IteratableChecksumValidator::IteratableChecksumValidator(const SingleFileDownloadContextHandle& dctx,
-							 const PieceStorageHandle& pieceStorage):
-  _dctx(dctx),
-  _pieceStorage(pieceStorage),
-  _currentOffset(0),
-  _ctx(0),
-  _logger(LogFactory::getInstance()),
-  _buffer(0) {}
-
-IteratableChecksumValidator::~IteratableChecksumValidator()
+IteratableChecksumValidator::IteratableChecksumValidator(
+    const std::shared_ptr<DownloadContext>& dctx,
+    const std::shared_ptr<PieceStorage>& pieceStorage)
+    : dctx_(dctx), pieceStorage_(pieceStorage), currentOffset_(0)
 {
-  delete [] _buffer;
 }
 
+IteratableChecksumValidator::~IteratableChecksumValidator() {}
+
 void IteratableChecksumValidator::validateChunk()
 {
-  if(!finished()) {
-    int32_t length = _pieceStorage->getDiskAdaptor()->readData(_buffer,
-							       BUFSIZE,
-							       _currentOffset);
-    _ctx->digestUpdate(_buffer, length);
-    _currentOffset += length;
-    if(finished()) {
-      string actualChecksum = Util::toHex((const unsigned char*)_ctx->digestFinal().c_str(), _ctx->digestLength());
-      if(_dctx->getChecksum() == actualChecksum) {
-	_pieceStorage->markAllPiecesDone();
-      } else {
-	BitfieldMan bitfield(_dctx->getPieceLength(), _dctx->getTotalLength());
-	_pieceStorage->setBitfield(bitfield.getBitfield(), bitfield.getBitfieldLength());
-      }
+  // Don't guard with !finished() to allow zero-length file to be
+  // verified.
+  std::array<unsigned char, 4_k> buf;
+  size_t length = pieceStorage_->getDiskAdaptor()->readDataDropCache(
+      buf.data(), buf.size(), currentOffset_);
+  ctx_->update(buf.data(), length);
+  currentOffset_ += length;
+  if (finished()) {
+    std::string actualDigest = ctx_->digest();
+    if (dctx_->getDigest() == actualDigest) {
+      pieceStorage_->markAllPiecesDone();
+      dctx_->setChecksumVerified(true);
+    }
+    else {
+      A2_LOG_INFO(fmt("Checksum validation failed. expected=%s, actual=%s",
+                      util::toHex(dctx_->getDigest()).c_str(),
+                      util::toHex(actualDigest).c_str()));
+      BitfieldMan bitfield(dctx_->getPieceLength(), dctx_->getTotalLength());
+      pieceStorage_->setBitfield(bitfield.getBitfield(),
+                                 bitfield.getBitfieldLength());
     }
   }
 }
 
 bool IteratableChecksumValidator::finished() const
 {
-  if(_currentOffset >= _dctx->getTotalLength()) {
-    _pieceStorage->getDiskAdaptor()->disableDirectIO();
+  if (currentOffset_ >= dctx_->getTotalLength()) {
     return true;
-  } else {
+  }
+  else {
     return false;
   }
 }
 
 int64_t IteratableChecksumValidator::getTotalLength() const
 {
-  return _dctx->getTotalLength();
+  return dctx_->getTotalLength();
 }
 
 void IteratableChecksumValidator::init()
 {
-#ifdef HAVE_POSIX_MEMALIGN
-  _buffer = (unsigned char*)Util::allocateAlignedMemory(ALIGNMENT, BUFSIZE);
-#else
-  _buffer = new unsigned char[BUFSIZE];
-#endif // HAVE_POSIX_MEMALIGN
-  _pieceStorage->getDiskAdaptor()->enableDirectIO();
-  _currentOffset = 0;
-  _ctx = new MessageDigestContext();
-  _ctx->trySetAlgo(_dctx->getChecksumHashAlgo());
-  _ctx->digestInit();
+  currentOffset_ = 0;
+  ctx_ = MessageDigest::create(dctx_->getHashType());
 }
+
+} // namespace aria2
