@@ -32,14 +32,14 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_MSE_HANDSHAKE_H_
-#define _D_MSE_HANDSHAKE_H_
+#ifndef D_MSE_HANDSHAKE_H
+#define D_MSE_HANDSHAKE_H
 
 #include "common.h"
 
 #include <vector>
+#include <memory>
 
-#include "SharedHandle.h"
 #include "BtConstants.h"
 #include "SocketBuffer.h"
 #include "Command.h"
@@ -47,12 +47,11 @@
 namespace aria2 {
 
 class Option;
-class Logger;
 class SocketCore;
 class DHKeyExchange;
 class ARC4Encryptor;
-class ARC4Decryptor;
 class DownloadContext;
+class MessageDigest;
 
 class MSEHandshake {
 public:
@@ -68,23 +67,20 @@ public:
     CRYPTO_ARC4 = 0x02u
   };
 
+  static constexpr size_t VC_LENGTH = 8U;
+
 private:
-  static const size_t PRIME_BITS = 768;
-  
-  static const size_t KEY_LENGTH = (PRIME_BITS+7)/8;
-  
-  static const size_t MAX_PAD_LENGTH = 512;
-
-  static const size_t VC_LENGTH = 8;
-
-  static const size_t CRYPTO_BITFIELD_LENGTH = 4;
-
-  static const size_t MAX_BUFFER_LENGTH = 6*1024;
+  static constexpr size_t PRIME_BITS = 768U;
+  static constexpr size_t KEY_LENGTH = (PRIME_BITS + 7U) / 8U;
+  // The largest buffering occurs when receiver receives step2
+  // handshake.  We believe that IA is less than or equal to
+  // BtHandshakeMessage::MESSAGE_LENGTH
+  static constexpr size_t MAX_BUFFER_LENGTH = 636U;
 
   cuid_t cuid_;
-  SharedHandle<SocketCore> socket_;
+  std::shared_ptr<SocketCore> socket_;
+  bool wantRead_;
   const Option* option_;
-  Logger* logger_;
 
   unsigned char rbuf_[MAX_BUFFER_LENGTH];
   size_t rbufLength_;
@@ -92,9 +88,9 @@ private:
   SocketBuffer socketBuffer_;
 
   CRYPTO_TYPE negotiatedCryptoType_;
-  DHKeyExchange* dh_;
-  SharedHandle<ARC4Encryptor> encryptor_;
-  SharedHandle<ARC4Decryptor> decryptor_;
+  std::unique_ptr<DHKeyExchange> dh_;
+  std::unique_ptr<ARC4Encryptor> encryptor_;
+  std::unique_ptr<ARC4Encryptor> decryptor_;
   unsigned char infoHash_[INFO_HASH_LENGTH];
   unsigned char secret_[KEY_LENGTH];
   bool initiator_;
@@ -102,15 +98,10 @@ private:
   size_t markerIndex_;
   uint16_t padLength_;
   uint16_t iaLength_;
-  unsigned char* ia_;
+  std::vector<unsigned char> ia_;
+  std::unique_ptr<MessageDigest> sha1_;
 
-  static const unsigned char* PRIME;
-
-  static const unsigned char* GENERATOR;
-
-  static const unsigned char VC[VC_LENGTH];
-
-  void encryptAndSendData(const unsigned char* data, size_t length);
+  void encryptAndSendData(std::vector<unsigned char> data);
 
   void createReq1Hash(unsigned char* md) const;
 
@@ -123,32 +114,45 @@ private:
     return decodeLength16(reinterpret_cast<const unsigned char*>(buffer));
   }
 
-  uint16_t verifyPadLength(const unsigned char* padlenbuf,
-                           const char* padName);
+  uint16_t verifyPadLength(const unsigned char* padlenbuf, const char* padName);
 
-  void verifyVC(const unsigned char* vcbuf);
+  void verifyVC(unsigned char* vcbuf);
 
   void verifyReq1Hash(const unsigned char* req1buf);
 
-  size_t receiveNBytes(size_t bytes);
+  void shiftBuffer(size_t offset);
 
 public:
-  MSEHandshake(cuid_t cuid, const SharedHandle<SocketCore>& socket,
+  MSEHandshake(cuid_t cuid, const std::shared_ptr<SocketCore>& socket,
                const Option* op);
 
   ~MSEHandshake();
-  
+
   HANDSHAKE_TYPE identifyHandshakeType();
 
   void initEncryptionFacility(bool initiator);
 
-  bool sendPublicKey();
+  // Reads data from Socket. If EOF is reached, throws
+  // RecoverableException.
+  void read();
+
+  // Sends pending data in the send buffer. Returns true if all data
+  // is sent. Otherwise returns false.
+  bool send();
+
+  bool getWantRead() const { return wantRead_; }
+
+  void setWantRead(bool wantRead) { wantRead_ = wantRead; }
+
+  bool getWantWrite() const;
+
+  void sendPublicKey();
 
   bool receivePublicKey();
 
   void initCipher(const unsigned char* infoHash);
 
-  bool sendInitiatorStep2();
+  void sendInitiatorStep2();
 
   bool findInitiatorVCMarker();
 
@@ -158,58 +162,45 @@ public:
 
   bool findReceiverHashMarker();
 
-  bool receiveReceiverHashAndPadCLength
-  (const std::vector<SharedHandle<DownloadContext> >& downloadContexts);
+  bool receiveReceiverHashAndPadCLength(
+      const std::vector<std::shared_ptr<DownloadContext>>& downloadContexts);
 
   bool receiveReceiverIALength();
 
   bool receiveReceiverIA();
 
-  bool sendReceiverStep2();
+  void sendReceiverStep2();
 
   // returns plain text IA
-  const unsigned char* getIA() const
-  {
-    return ia_;
-  }
+  const unsigned char* getIA() const { return ia_.data(); }
 
-  size_t getIALength() const
-  {
-    return iaLength_;
-  }
+  size_t getIALength() const { return iaLength_; }
 
-  const unsigned char* getInfoHash() const
-  {
-    return infoHash_;
-  }
+  const unsigned char* getInfoHash() const { return infoHash_; }
 
-  CRYPTO_TYPE getNegotiatedCryptoType() const
-  {
-    return negotiatedCryptoType_;
-  }
+  CRYPTO_TYPE getNegotiatedCryptoType() const { return negotiatedCryptoType_; }
 
-  const SharedHandle<ARC4Encryptor>& getEncryptor() const
+  const std::unique_ptr<ARC4Encryptor>& getEncryptor() const
   {
     return encryptor_;
   }
 
-  const SharedHandle<ARC4Decryptor>& getDecryptor() const
+  const std::unique_ptr<ARC4Encryptor>& getDecryptor() const
   {
     return decryptor_;
   }
 
-  const unsigned char* getBuffer() const
-  {
-    return rbuf_;
-  }
+  std::unique_ptr<ARC4Encryptor> popEncryptor();
 
-  size_t getBufferLength() const
-  {
-    return rbufLength_;
-  }
+  std::unique_ptr<ARC4Encryptor> popDecryptor();
 
+  const unsigned char* getBuffer() const { return rbuf_; }
+
+  unsigned char* getBuffer() { return rbuf_; }
+
+  size_t getBufferLength() const { return rbufLength_; }
 };
 
 } // namespace aria2
 
-#endif // _D_MSE_HANDSHAKE_H_
+#endif // D_MSE_HANDSHAKE_H
