@@ -39,8 +39,7 @@
 
 #include <string>
 #include <vector>
-
-#include "SharedHandle.h"
+#include <memory>
 
 namespace aria2 {
 
@@ -57,65 +56,52 @@ private:
   std::string login_;
   std::string password_;
   std::string account_;
+
 public:
   Authenticator();
 
-  Authenticator(const std::string& machine,
-                const std::string& login,
-                const std::string& password,
-                const std::string& account);
+  Authenticator(std::string machine, std::string login, std::string password,
+                std::string account);
 
   virtual ~Authenticator();
 
-  virtual bool match(const std::string& hostname) const;
+  virtual bool match(const std::string& hostname) const CXX11_OVERRIDE;
 
-  const std::string& getMachine() const
-  {
-    return machine_;
-  }
+  const std::string& getMachine() const { return machine_; }
 
-  void setMachine(const std::string& machine);
+  void setMachine(std::string machine);
 
-  const std::string& getLogin() const
-  {
-    return login_;
-  }
+  const std::string& getLogin() const { return login_; }
 
-  void setLogin(const std::string& login);
+  void setLogin(std::string login);
 
-  const std::string& getPassword() const
-  {
-    return password_;
-  }
+  const std::string& getPassword() const { return password_; }
 
-  void setPassword(const std::string& password);
+  void setPassword(std::string password);
 
-  const std::string& getAccount() const
-  {
-    return account_;
-  }
+  const std::string& getAccount() const { return account_; }
 
-  void setAccount(const std::string& account);
+  void setAccount(std::string account);
 };
 
 class DefaultAuthenticator : public Authenticator {
 public:
   DefaultAuthenticator();
 
-  DefaultAuthenticator(const std::string& login,
-                       const std::string& password,
-                       const std::string& account);
+  DefaultAuthenticator(std::string login, std::string password,
+                       std::string account);
 
   virtual ~DefaultAuthenticator();
 
-  virtual bool match(const std::string& hostname) const;
+  virtual bool match(const std::string& hostname) const CXX11_OVERRIDE;
 };
 
 class Netrc {
 private:
-  std::vector<SharedHandle<Authenticator> > authenticators_;
+  std::vector<std::unique_ptr<Authenticator>> authenticators_;
+
+  void storeAuthenticator(std::unique_ptr<Authenticator> authenticator);
 
-  void storeAuthenticator(const SharedHandle<Authenticator>& authenticator);
 public:
   Netrc();
 
@@ -123,15 +109,11 @@ public:
 
   void parse(const std::string& path);
 
-  SharedHandle<Authenticator> findAuthenticator
-  (const std::string& hostname) const;
+  const Authenticator* findAuthenticator(const std::string& hostname) const;
 
-  const std::vector<SharedHandle<Authenticator> >& getAuthenticators() const
-  {
-    return authenticators_;
-  }
+  const std::vector<std::unique_ptr<Authenticator>>& getAuthenticators() const;
 
-  void addAuthenticator(const SharedHandle<Authenticator>& authenticator);
+  void addAuthenticator(std::unique_ptr<Authenticator> authenticator);
 };
 
 } // namespace aria2
