@@ -39,8 +39,8 @@
 
 #include <string>
 #include <vector>
+#include <memory>
 
-#include "SharedHandle.h"
 #include "TimerA2.h"
 #include "Command.h"
 
@@ -51,6 +51,7 @@ class Piece;
 class Peer;
 #endif // ENABLE_BITTORRENT
 class DiskAdaptor;
+class WrDiskCache;
 
 class PieceStorage {
 public:
@@ -61,31 +62,29 @@ public:
    * Returns true if the peer has a piece that localhost doesn't have.
    * Otherwise returns false.
    */
-  virtual bool hasMissingPiece(const SharedHandle<Peer>& peer) = 0;
+  virtual bool hasMissingPiece(const std::shared_ptr<Peer>& peer) = 0;
 
-   // Stores pieces that the peer has but localhost doesn't.  Those
-   // pieces will be marked "used" status in order to prevent other
-   // command from get the same piece. But in end game mode, same
-   // piece may be got by several commands. This function stores N
-   // pieces where the sum of missing block of first N-1 pieces is
-   // less than minMissingBlocks and the sum of missing block of N
-   // pieces is greater than or equal to minMissingBlocks. If there is
-   // M missing pieces left where M < N, M pieces are stored.
-  virtual void getMissingPiece
-  (std::vector<SharedHandle<Piece> >& pieces,
-   size_t minMissingBlocks,
-   const SharedHandle<Peer>& peer,
-   cuid_t cuid) = 0;
+  // Stores pieces that the peer has but localhost doesn't.  Those
+  // pieces will be marked "used" status in order to prevent other
+  // command from get the same piece. But in end game mode, same
+  // piece may be got by several commands. This function stores N
+  // pieces where the sum of missing block of first N-1 pieces is
+  // less than minMissingBlocks and the sum of missing block of N
+  // pieces is greater than or equal to minMissingBlocks. If there is
+  // M missing pieces left where M < N, M pieces are stored.
+  virtual void getMissingPiece(std::vector<std::shared_ptr<Piece>>& pieces,
+                               size_t minMissingBlocks,
+                               const std::shared_ptr<Peer>& peer,
+                               cuid_t cuid) = 0;
 
   // Same as getMissingPiece(pieces, minMissingBlocks, peer), but the
   // indexes in excludedIndexes are excluded.
-  virtual void getMissingPiece
-  (std::vector<SharedHandle<Piece> >& pieces,
-   size_t minMissingBlocks,
-   const SharedHandle<Peer>& peer,
-   const std::vector<size_t>& excludedIndexes,
-   cuid_t cuid) = 0;
-  
+  virtual void getMissingPiece(std::vector<std::shared_ptr<Piece>>& pieces,
+                               size_t minMissingBlocks,
+                               const std::shared_ptr<Peer>& peer,
+                               const std::vector<size_t>& excludedIndexes,
+                               cuid_t cuid) = 0;
+
   // Stores pieces that the peer has but localhost doesn't.  Only
   // pieces that declared as "fast" are stored.  Those pieces stored
   // will be marked "used" status in order to prevent other command
@@ -95,20 +94,18 @@ public:
   // minMissingBlocks and the sum of missing block of N pieces is
   // greater than or equal to minMissingBlocks. If there is M missing
   // pieces left where M < N, M pieces are stored.
-  virtual void getMissingFastPiece
-  (std::vector<SharedHandle<Piece> >& pieces,
-   size_t minMissingBlocks,
-   const SharedHandle<Peer>& peer,
-   cuid_t cuid) = 0;
+  virtual void getMissingFastPiece(std::vector<std::shared_ptr<Piece>>& pieces,
+                                   size_t minMissingBlocks,
+                                   const std::shared_ptr<Peer>& peer,
+                                   cuid_t cuid) = 0;
 
   // Same as getMissingFastPiece(pieces, minMissingBlocks, peer), but
   // the indexes in excludedIndexes are excluded.
-  virtual void getMissingFastPiece
-  (std::vector<SharedHandle<Piece> >& pieces,
-   size_t minMissingBlocks,
-   const SharedHandle<Peer>& peer,
-   const std::vector<size_t>& excludedIndexes,
-   cuid_t cuid) = 0;
+  virtual void getMissingFastPiece(std::vector<std::shared_ptr<Piece>>& pieces,
+                                   size_t minMissingBlocks,
+                                   const std::shared_ptr<Peer>& peer,
+                                   const std::vector<size_t>& excludedIndexes,
+                                   cuid_t cuid) = 0;
 
   /**
    * Returns a piece that the peer has but localhost doesn't.
@@ -116,17 +113,17 @@ public:
    * from get the same piece. But in end game mode, same piece may be returned
    * to several commands.
    */
-  virtual SharedHandle<Piece>
-  getMissingPiece(const SharedHandle<Peer>& peer, cuid_t cuid) = 0;
+  virtual std::shared_ptr<Piece>
+  getMissingPiece(const std::shared_ptr<Peer>& peer, cuid_t cuid) = 0;
 
   /**
-   * Same as getMissingPiece(const SharedHandle<Peer>& peer), but the indexes in
+   * Same as getMissingPiece(const std::shared_ptr<Peer>& peer), but the indexes
+   * in
    * excludedIndexes are excluded.
    */
-  virtual SharedHandle<Piece> getMissingPiece
-  (const SharedHandle<Peer>& peer,
-   const std::vector<size_t>& excludedIndexes,
-   cuid_t cuid) = 0;
+  virtual std::shared_ptr<Piece>
+  getMissingPiece(const std::shared_ptr<Peer>& peer,
+                  const std::vector<size_t>& excludedIndexes, cuid_t cuid) = 0;
 #endif // ENABLE_BITTORRENT
 
   // Returns true if there is at least one missing and unused piece.
@@ -136,11 +133,9 @@ public:
    * Returns a missing piece if available. Otherwise returns 0;
    * If ignoreBitfield is set, indexes of true bit are excluded.
    */
-  virtual SharedHandle<Piece> getMissingPiece
-  (size_t minSplitSize,
-   const unsigned char* ignoreBitfield,
-   size_t length,
-   cuid_t cuid) = 0;
+  virtual std::shared_ptr<Piece>
+  getMissingPiece(size_t minSplitSize, const unsigned char* ignoreBitfield,
+                  size_t length, cuid_t cuid) = 0;
 
   /**
    * Returns a missing piece whose index is index.
@@ -148,13 +143,13 @@ public:
    * then returns 0.
    * Also returns 0 if any of missing piece is not available.
    */
-  virtual SharedHandle<Piece> getMissingPiece(size_t index, cuid_t cuid) = 0;
+  virtual std::shared_ptr<Piece> getMissingPiece(size_t index, cuid_t cuid) = 0;
 
   /**
    * Returns the piece denoted by index.
    * No status of the piece is changed in this method.
    */
-  virtual SharedHandle<Piece> getPiece(size_t index) = 0;
+  virtual std::shared_ptr<Piece> getPiece(size_t index) = 0;
 
   /**
    * Marks the piece whose index is index as missing.
@@ -164,12 +159,13 @@ public:
   /**
    * Tells that the download of the specfied piece completes.
    */
-  virtual void completePiece(const SharedHandle<Piece>& piece) = 0;
+  virtual void completePiece(const std::shared_ptr<Piece>& piece) = 0;
 
   /**
    * Tells that the download of the specified piece is canceled.
    */
-  virtual void cancelPiece(const SharedHandle<Piece>& piece, cuid_t cuid) = 0;
+  virtual void cancelPiece(const std::shared_ptr<Piece>& piece,
+                           cuid_t cuid) = 0;
 
   /**
    * Returns true if the specified piece is already downloaded.
@@ -179,18 +175,18 @@ public:
 
   virtual bool isPieceUsed(size_t index) = 0;
 
-  virtual uint64_t getTotalLength() = 0;
+  virtual int64_t getTotalLength() = 0;
 
-  virtual uint64_t getFilteredTotalLength() = 0;
+  virtual int64_t getFilteredTotalLength() = 0;
 
-  virtual uint64_t getCompletedLength() = 0;
+  virtual int64_t getCompletedLength() = 0;
 
-  virtual uint64_t getFilteredCompletedLength() = 0;
+  virtual int64_t getFilteredCompletedLength() = 0;
 
   virtual void setupFileFilter() = 0;
-  
+
   virtual void clearFileFilter() = 0;
-  
+
   /**
    * Returns true if download has completed.
    * If file filter is enabled, then returns true if those files have
@@ -214,7 +210,7 @@ public:
 
   virtual void setBitfield(const unsigned char* bitfield,
                            size_t bitfieldLength) = 0;
-  
+
   virtual size_t getBitfieldLength() = 0;
 
   virtual bool isSelectiveDownloadingMode() = 0;
@@ -226,29 +222,35 @@ public:
   // TODO We can remove this.
   virtual void setEndGamePieceNum(size_t num) = 0;
 
-  virtual SharedHandle<DiskAdaptor> getDiskAdaptor() = 0;
-  
-  virtual size_t getPieceLength(size_t index) = 0;
+  virtual std::shared_ptr<DiskAdaptor> getDiskAdaptor() = 0;
+
+  virtual WrDiskCache* getWrDiskCache() = 0;
+
+  // Flushes write disk cache for in-flight piece and evicts them.
+  virtual void flushWrDiskCacheEntry() = 0;
+
+  virtual int32_t getPieceLength(size_t index) = 0;
 
   /**
    * Adds piece index to advertise to other commands. They send have message
    * based on this information.
    */
-  virtual void advertisePiece(cuid_t cuid, size_t index) = 0;
+  virtual void advertisePiece(cuid_t cuid, size_t index,
+                              Timer registerdTime) = 0;
 
   /**
-   * indexes is filled with piece index which is not advertised by the caller
-   * command and newer than lastCheckTime.
+   * indexes is filled with piece index which is not advertised by the
+   * caller command and newer than lastHaveIndex.
    */
-  virtual void getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
-                                         cuid_t myCuid,
-                                         const Timer& lastCheckTime) = 0;
+  virtual uint64_t getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
+                                             cuid_t myCuid,
+                                             uint64_t lastHaveIndex) = 0;
 
   /**
-   * Removes have entry if specified seconds have elapsed since its
-   * registration.
+   * Removes have entry if its registeredTime is at least as old as
+   * expiry.
    */
-  virtual void removeAdvertisedPiece(time_t elapsed) = 0;
+  virtual void removeAdvertisedPiece(const Timer& expiry) = 0;
 
   /**
    * Sets all bits in bitfield to 1.
@@ -258,14 +260,15 @@ public:
   /**
    * Sets all bits in bitfield(0 to length) to 1.
    */
-  virtual void markPiecesDone(uint64_t length) = 0;
+  virtual void markPiecesDone(int64_t length) = 0;
 
   virtual void
-  addInFlightPiece(const std::vector<SharedHandle<Piece> >& pieces) = 0;
+  addInFlightPiece(const std::vector<std::shared_ptr<Piece>>& pieces) = 0;
 
   virtual size_t countInFlightPiece() = 0;
 
-  virtual void getInFlightPieces(std::vector<SharedHandle<Piece> >& pieces) = 0;
+  virtual void
+  getInFlightPieces(std::vector<std::shared_ptr<Piece>>& pieces) = 0;
 
   virtual void addPieceStats(size_t index) = 0;
 
@@ -288,8 +291,6 @@ public:
   virtual void onDownloadIncomplete() = 0;
 };
 
-typedef SharedHandle<PieceStorage> PieceStorageHandle;
-
 } // namespace aria2
 
 #endif // D_PIECE_STORAGE_H
