@@ -50,26 +50,21 @@ const std::string Request::METHOD_GET = "GET";
 
 const std::string Request::METHOD_HEAD = "HEAD";
 
-const std::string Request::PROTO_HTTP("http");
+const std::string Request::DEFAULT_FILE = "index.html";
 
-const std::string Request::PROTO_HTTPS("https");
-
-const std::string Request::PROTO_FTP("ftp");
-
-Request::Request():
-  port_(0), tryCount_(0),
-  redirectCount_(0),
-  supportsPersistentConnection_(true),
-  keepAliveHint_(false),
-  pipeliningHint_(false),
-  maxPipelinedRequest_(1),
-  method_(METHOD_GET),
-  hasPassword_(false),
-  ipv6LiteralAddress_(false),
-  removalRequested_(false),
-  connectedPort_(0),
-  wakeTime_(global::wallclock())
-{}
+Request::Request()
+    : method_(METHOD_GET),
+      tryCount_(0),
+      redirectCount_(0),
+      supportsPersistentConnection_(true),
+      keepAliveHint_(false),
+      pipeliningHint_(false),
+      maxPipelinedRequest_(1),
+      removalRequested_(false),
+      connectedPort_(0),
+      wakeTime_(global::wallclock())
+{
+}
 
 Request::~Request() {}
 
@@ -77,22 +72,24 @@ namespace {
 std::string removeFragment(const std::string& uri)
 {
   std::string::size_type sharpIndex = uri.find("#");
-  if(sharpIndex == std::string::npos) {
+  if (sharpIndex == std::string::npos) {
     return uri;
-  } else {
+  }
+  else {
     return uri.substr(0, sharpIndex);
   }
 }
 } // namespace
 
-bool Request::setUri(const std::string& uri) {
+bool Request::setUri(const std::string& uri)
+{
   supportsPersistentConnection_ = true;
   uri_ = uri;
   return parseUri(uri_);
 }
 
-bool Request::resetUri() {
-  previousUri_ = referer_;
+bool Request::resetUri()
+{
   supportsPersistentConnection_ = true;
   setConnectedAddrInfo(A2STR::NIL, A2STR::NIL, 0);
   return parseUri(uri_);
@@ -100,86 +97,102 @@ bool Request::resetUri() {
 
 void Request::setReferer(const std::string& uri)
 {
-  referer_ = previousUri_ = removeFragment(uri);
+  referer_ = removeFragment(uri);
 }
 
-bool Request::redirectUri(const std::string& uri) {
+bool Request::redirectUri(const std::string& uri)
+{
   supportsPersistentConnection_ = true;
   ++redirectCount_;
+  if (uri.empty()) {
+    return false;
+  }
   std::string redirectedUri;
-  if(uri.find("://") == std::string::npos) {
-    // rfc2616 requires absolute URI should be provided by Location header
-    // field, but some servers don't obey this rule.
-    if(util::startsWith(uri, "/")) {
-      // abosulute path
-      redirectedUri = strconcat(protocol_, "://", host_, uri);
-    } else {
-      // relative path
-      redirectedUri = strconcat(protocol_, "://", host_, dir_, "/", uri);
+  if (util::startsWith(uri, "//")) {
+    // Network-path reference (according to RFC 3986, Section 4.2)
+    // Just complement current protocol.
+    redirectedUri = getProtocol();
+    redirectedUri += ":";
+    redirectedUri += uri;
+  }
+  else {
+    std::string::size_type schemeEnd = uri.find("://");
+    bool absUri;
+    if (schemeEnd == std::string::npos) {
+      absUri = false;
+    }
+    else {
+      absUri = true;
+      // Check that scheme is acceptable one.
+      for (size_t i = 0; i < schemeEnd; ++i) {
+        char c = uri[i];
+        if (!util::isAlpha(c) && !util::isDigit(c) && c != '+' && c != '-' &&
+            c != '.') {
+          absUri = false;
+          break;
+        }
+      }
+    }
+    if (absUri) {
+      redirectedUri = uri;
+    }
+    else {
+      // rfc2616 requires absolute URI should be provided by Location header
+      // field, but some servers don't obey this rule.
+      // UPDATE: draft-ietf-httpbis-p2-semantics-18 now allows this.
+      redirectedUri = uri::joinUri(currentUri_, uri);
     }
-  } else {
-    redirectedUri = uri;
   }
   return parseUri(redirectedUri);
 }
 
-bool Request::parseUri(const std::string& srcUri) {
+bool Request::parseUri(const std::string& srcUri)
+{
   currentUri_ = removeFragment(srcUri);
   uri::UriStruct us;
-  if(uri::parse(us, currentUri_)) {
-    protocol_.swap(us.protocol);
-    host_.swap(us.host);
-    port_ = us.port;
-    dir_.swap(us.dir);
-    file_.swap(us.file);
-    query_.swap(us.query);
-    username_.swap(us.username);
-    password_.swap(us.password);
-    hasPassword_ = us.hasPassword;
-    ipv6LiteralAddress_ = us.ipv6LiteralAddress;
+  if (uri::parse(us, currentUri_)) {
+    us_.swap(us);
     return true;
-  } else {
+  }
+  else {
     return false;
   }
 }
 
-void Request::resetRedirectCount()
-{
-  redirectCount_ = 0;
-}
-  
-void Request::setMaxPipelinedRequest(unsigned int num)
-{
-  maxPipelinedRequest_ = num;
-}
+void Request::resetRedirectCount() { redirectCount_ = 0; }
 
-const SharedHandle<PeerStat>& Request::initPeerStat()
+void Request::setMaxPipelinedRequest(int num) { maxPipelinedRequest_ = num; }
+
+const std::shared_ptr<PeerStat>& Request::initPeerStat()
 {
   // Use host and protocol in original URI, because URI selector
   // selects URI based on original URI, not redirected one.
-  uri::UriStruct us;
-  bool v = uri::parse(us, uri_);
-  assert(v);
-  peerStat_.reset(new PeerStat(0, us.host, us.protocol));
+  uri_split_result us;
+  int v = uri_split(&us, uri_.c_str());
+  assert(v == 0);
+  std::string host = uri::getFieldString(us, USR_HOST, uri_.c_str());
+  std::string protocol = uri::getFieldString(us, USR_SCHEME, uri_.c_str());
+  peerStat_ = std::make_shared<PeerStat>(0, host, protocol);
   return peerStat_;
 }
 
 std::string Request::getURIHost() const
 {
-  if(isIPv6LiteralAddress()) {
-    return strconcat("[", getHost(), "]");
-  } else {
+  if (isIPv6LiteralAddress()) {
+    std::string s = "[";
+    s += getHost();
+    s += "]";
+    return s;
+  }
+  else {
     return getHost();
   }
 }
 
-void Request::setMethod(const std::string& method)
-{
-  method_ = method;
-}
+void Request::setMethod(const std::string& method) { method_ = method; }
 
-void Request::setConnectedAddrInfo
-(const std::string& hostname, const std::string& addr, uint16_t port)
+void Request::setConnectedAddrInfo(const std::string& hostname,
+                                   const std::string& addr, uint16_t port)
 {
   connectedHostname_ = hostname;
   connectedAddr_ = addr;
