@@ -40,13 +40,15 @@
 #include <string>
 #include <algorithm>
 #include <vector>
+#include <memory>
+#include <utility>
 
-#include "SharedHandle.h"
 #include "TransferStat.h"
 #include "TimeA2.h"
 #include "Request.h"
 #include "error_code.h"
 #include "MetadataInfo.h"
+#include "GroupId.h"
 
 namespace aria2 {
 
@@ -73,30 +75,75 @@ class BtRuntime;
 class PeerStorage;
 #endif // ENABLE_BITTORRENT
 
-typedef int64_t a2_gid_t;
-
 class RequestGroup {
 public:
-  enum HaltReason {
-    NONE,
-    SHUTDOWN_SIGNAL,
-    USER_REQUEST
-  };
+  enum HaltReason { NONE, SHUTDOWN_SIGNAL, USER_REQUEST };
   enum State {
     // Waiting in the reserved queue
     STATE_WAITING,
     // Download has begun
     STATE_ACTIVE
   };
-private:
-  static a2_gid_t gidCounter_;
 
-  a2_gid_t gid_;
+private:
+  // If this download is a part of another download(for example,
+  // downloading torrent file described in Metalink file), this field
+  // has the GID of parent RequestGroup. 0 means this is a parent
+  // RequestGroup.
+  a2_gid_t belongsToGID_;
+
+  std::shared_ptr<GroupId> gid_;
+
+  std::shared_ptr<Option> option_;
+
+  // options applied on restart
+  std::shared_ptr<Option> pendingOption_;
+
+  std::shared_ptr<SegmentMan> segmentMan_;
+
+  std::shared_ptr<DownloadContext> downloadContext_;
+
+  std::shared_ptr<PieceStorage> pieceStorage_;
+
+  std::shared_ptr<BtProgressInfoFile> progressInfoFile_;
+
+  std::shared_ptr<DiskWriterFactory> diskWriterFactory_;
+
+  std::shared_ptr<Dependency> dependency_;
+
+  std::unique_ptr<URISelector> uriSelector_;
+
+  std::shared_ptr<MetadataInfo> metadataInfo_;
+
+  RequestGroupMan* requestGroupMan_;
+
+#ifdef ENABLE_BITTORRENT
+  BtRuntime* btRuntime_;
+
+  PeerStorage* peerStorage_;
+#endif // ENABLE_BITTORRENT
+
+  // If this download generates another downloads when completed(for
+  // example, downloads generated by PostDownloadHandler), this field
+  // has the GID of generated RequestGroups. empty list means there is
+  // no such RequestGroup.
+  std::vector<a2_gid_t> followedByGIDs_;
+  // This is a reverse link against followedByGIDs_.  For example, a
+  // download included in followedByGIDs_ has this download's GID in
+  // followingGID_.
+  a2_gid_t followingGID_;
+
+  std::vector<const PreDownloadHandler*> preDownloadHandlers_;
+
+  std::vector<const PostDownloadHandler*> postDownloadHandlers_;
+
+  Time lastModifiedTime_;
+
+  // Timeout used for HTTP/FTP downloads.
+  std::chrono::seconds timeout_;
 
   int state_;
 
-  SharedHandle<Option> option_;
-
   int numConcurrentCommand_;
 
   /**
@@ -108,20 +155,22 @@ private:
 
   int numCommand_;
 
-  SharedHandle<SegmentMan> segmentMan_;
+  int fileNotFoundCount_;
 
-  SharedHandle<DownloadContext> downloadContext_;
+  int maxDownloadSpeedLimit_;
 
-  SharedHandle<PieceStorage> pieceStorage_;
+  int maxUploadSpeedLimit_;
+
+  int resumeFailureCount_;
+
+  HaltReason haltReason_;
+
+  error_code::Value lastErrorCode_;
+
+  std::string lastErrorMessage_;
 
   bool saveControlFile_;
 
-  SharedHandle<BtProgressInfoFile> progressInfoFile_;
-
-  SharedHandle<DiskWriterFactory> diskWriterFactory_;
-
-  SharedHandle<Dependency> dependency_;
-
   bool fileAllocationEnabled_;
 
   bool preLocalFileCheckEnabled_;
@@ -130,58 +179,18 @@ private:
 
   bool forceHaltRequested_;
 
-  HaltReason haltReason_;
-
   bool pauseRequested_;
 
-  std::vector<SharedHandle<PreDownloadHandler> > preDownloadHandlers_;
-
-  std::vector<SharedHandle<PostDownloadHandler> > postDownloadHandlers_;
-
-  std::vector<std::string> acceptTypes_;
-
-  SharedHandle<URISelector> uriSelector_;
-
-  Time lastModifiedTime_;
-
-  int fileNotFoundCount_;
-
-  // Timeout used for HTTP/FTP downloads.
-  time_t timeout_;
-
-#ifdef ENABLE_BITTORRENT
-  BtRuntime* btRuntime_;
-
-  PeerStorage* peerStorage_;
-#endif // ENABLE_BITTORRENT
+  // restartRequested_ indicates that this download should be
+  // restarted.  Usually, it is used with pauseRequested_ to stop
+  // download first.
+  bool restartRequested_;
 
   // This flag just indicates that the downloaded file is not saved disk but
   // just sits in memory.
   bool inMemoryDownload_;
 
-  int maxDownloadSpeedLimit_;
-
-  int maxUploadSpeedLimit_;
-
-  error_code::Value lastErrorCode_;
-
-  // If this download generates another downloads when completed(for
-  // example, downloads generated by PostDownloadHandler), this field
-  // has the GID of generated RequestGroups. empty list means there is
-  // no such RequestGroup.
-  std::vector<a2_gid_t> followedByGIDs_;
-
-  // If this download is a part of another download(for example,
-  // downloading torrent file described in Metalink file), this field
-  // has the GID of parent RequestGroup. 0 means this is a parent
-  // RequestGroup.
-  a2_gid_t belongsToGID_;
-
-  SharedHandle<MetadataInfo> metadataInfo_;
-
-  RequestGroupMan* requestGroupMan_;
-
-  int resumeFailureCount_;
+  bool seedOnly_;
 
   void validateFilename(const std::string& expectedFilename,
                         const std::string& actualFilename) const;
@@ -190,45 +199,47 @@ private:
 
   void initializePostDownloadHandler();
 
-  bool tryAutoFileRenaming();
+  void tryAutoFileRenaming();
 
   // Returns the result code of this RequestGroup.  If the download
   // finished, then returns error_code::FINISHED.  If the
   // download didn't finish and error result is available in
   // _uriResults, then last result code is returned.  Otherwise
   // returns error_code::UNKNOWN_ERROR.
-  error_code::Value downloadResult() const;
+  std::pair<error_code::Value, std::string> downloadResult() const;
 
-  void removeDefunctControlFile
-  (const SharedHandle<BtProgressInfoFile>& progressInfoFile);
+  void removeDefunctControlFile(
+      const std::shared_ptr<BtProgressInfoFile>& progressInfoFile);
 
 public:
-  RequestGroup(const SharedHandle<Option>& option);
+  RequestGroup(const std::shared_ptr<GroupId>& gid,
+               const std::shared_ptr<Option>& option);
 
   ~RequestGroup();
 
   bool isCheckIntegrityReady();
 
-  const SharedHandle<SegmentMan>& getSegmentMan() const
+  const std::shared_ptr<SegmentMan>& getSegmentMan() const
   {
     return segmentMan_;
   }
 
-  SharedHandle<CheckIntegrityEntry> createCheckIntegrityEntry();
+  std::unique_ptr<CheckIntegrityEntry> createCheckIntegrityEntry();
 
   // Returns first bootstrap commands to initiate a download.
   // If this is HTTP/FTP download and file size is unknown, only 1 command
   // (usually, HttpInitiateConnection or FtpInitiateConnection) will be created.
-  void createInitialCommand(std::vector<Command*>& commands,
+  void createInitialCommand(std::vector<std::unique_ptr<Command>>& commands,
                             DownloadEngine* e);
 
-  void createNextCommandWithAdj(std::vector<Command*>& commands,
+  void createNextCommandWithAdj(std::vector<std::unique_ptr<Command>>& commands,
                                 DownloadEngine* e, int numAdj);
 
-  void createNextCommand(std::vector<Command*>& commands,
+  void createNextCommand(std::vector<std::unique_ptr<Command>>& commands,
                          DownloadEngine* e, int numCommand);
-  
-  void createNextCommand(std::vector<Command*>& commands, DownloadEngine* e);
+
+  void createNextCommand(std::vector<std::unique_ptr<Command>>& commands,
+                         DownloadEngine* e);
 
   bool downloadFinished() const;
 
@@ -242,6 +253,11 @@ public:
 
   int64_t getCompletedLength() const;
 
+  inline int64_t getPendingLength() const
+  {
+    return getTotalLength() - getCompletedLength();
+  }
+
   /**
    * Compares expected filename with specified actualFilename.
    * The expected filename refers to FileEntry::getBasename() of the first
@@ -254,40 +270,35 @@ public:
 
   void validateTotalLength(int64_t actualTotalLength) const;
 
-  void setNumConcurrentCommand(int num)
-  {
-    numConcurrentCommand_ = num;
-  }
+  void setNumConcurrentCommand(int num) { numConcurrentCommand_ = num; }
 
-  int getNumConcurrentCommand() const
-  {
-    return numConcurrentCommand_;
-  }
+  int getNumConcurrentCommand() const { return numConcurrentCommand_; }
 
-  a2_gid_t getGID() const
-  {
-    return gid_;
-  }
+  a2_gid_t getGID() const { return gid_->getNumericId(); }
+
+  const std::shared_ptr<GroupId>& getGroupId() const { return gid_; }
 
   TransferStat calculateStat() const;
 
-  const SharedHandle<DownloadContext>& getDownloadContext() const
+  const std::shared_ptr<DownloadContext>& getDownloadContext() const
   {
     return downloadContext_;
   }
 
   // This function also calls
   // downloadContext->setOwnerRequestGroup(this).
-  void setDownloadContext(const SharedHandle<DownloadContext>& downloadContext);
+  void
+  setDownloadContext(const std::shared_ptr<DownloadContext>& downloadContext);
 
-  const SharedHandle<PieceStorage>& getPieceStorage() const
+  const std::shared_ptr<PieceStorage>& getPieceStorage() const
   {
     return pieceStorage_;
   }
 
-  void setPieceStorage(const SharedHandle<PieceStorage>& pieceStorage);
+  void setPieceStorage(const std::shared_ptr<PieceStorage>& pieceStorage);
 
-  void setProgressInfoFile(const SharedHandle<BtProgressInfoFile>& progressInfoFile);
+  void setProgressInfoFile(
+      const std::shared_ptr<BtProgressInfoFile>& progressInfoFile);
 
   void increaseStreamCommand();
 
@@ -303,28 +314,21 @@ public:
 
   void decreaseNumCommand();
 
-  int getNumCommand() const
-  {
-    return numCommand_;
-  }
+  int getNumCommand() const { return numCommand_; }
 
-  // TODO is it better to move the following 2 methods to SingleFileDownloadContext?
-  void setDiskWriterFactory(const SharedHandle<DiskWriterFactory>& diskWriterFactory);
+  // TODO is it better to move the following 2 methods to
+  // SingleFileDownloadContext?
+  void setDiskWriterFactory(
+      const std::shared_ptr<DiskWriterFactory>& diskWriterFactory);
 
-  const SharedHandle<DiskWriterFactory>& getDiskWriterFactory() const
+  const std::shared_ptr<DiskWriterFactory>& getDiskWriterFactory() const
   {
     return diskWriterFactory_;
   }
 
-  void setFileAllocationEnabled(bool f)
-  {
-    fileAllocationEnabled_ = f;
-  }
+  void setFileAllocationEnabled(bool f) { fileAllocationEnabled_ = f; }
 
-  bool isFileAllocationEnabled() const
-  {
-    return fileAllocationEnabled_;
-  }
+  bool isFileAllocationEnabled() const { return fileAllocationEnabled_; }
 
   bool needsFileAllocation() const;
 
@@ -333,58 +337,49 @@ public:
    * if a file is already exists and control file exists etc.
    * Always open file with DiskAdaptor::initAndOpenFile()
    */
-  void setPreLocalFileCheckEnabled(bool f)
-  {
-    preLocalFileCheckEnabled_ = f;
-  }
+  void setPreLocalFileCheckEnabled(bool f) { preLocalFileCheckEnabled_ = f; }
 
-  bool isPreLocalFileCheckEnabled() const
-  {
-    return preLocalFileCheckEnabled_;
-  }
+  bool isPreLocalFileCheckEnabled() const { return preLocalFileCheckEnabled_; }
 
   void setHaltRequested(bool f, HaltReason = SHUTDOWN_SIGNAL);
 
   void setForceHaltRequested(bool f, HaltReason = SHUTDOWN_SIGNAL);
 
-  bool isHaltRequested() const
-  {
-    return haltRequested_;
-  }
+  bool isHaltRequested() const { return haltRequested_; }
 
-  bool isForceHaltRequested() const
-  {
-    return forceHaltRequested_;
-  }
+  bool isForceHaltRequested() const { return forceHaltRequested_; }
 
   void setPauseRequested(bool f);
 
-  bool isPauseRequested() const
-  {
-    return pauseRequested_;
-  }
+  bool isPauseRequested() const { return pauseRequested_; }
 
-  void dependsOn(const SharedHandle<Dependency>& dep);
+  void setRestartRequested(bool f);
+
+  bool isRestartRequested() const { return restartRequested_; }
+
+  void dependsOn(const std::shared_ptr<Dependency>& dep);
 
   bool isDependencyResolved();
 
   void releaseRuntimeResource(DownloadEngine* e);
 
-  void postDownloadProcessing(std::vector<SharedHandle<RequestGroup> >& groups);
+  void
+  postDownloadProcessing(std::vector<std::shared_ptr<RequestGroup>>& groups);
 
-  void addPostDownloadHandler(const SharedHandle<PostDownloadHandler>& handler);
+  void addPostDownloadHandler(const PostDownloadHandler* handler);
 
   void clearPostDownloadHandler();
 
   void preDownloadProcessing();
 
-  void addPreDownloadHandler(const SharedHandle<PreDownloadHandler>& handler);
+  void addPreDownloadHandler(const PreDownloadHandler* handler);
 
   void clearPreDownloadHandler();
 
-  void processCheckIntegrityEntry(std::vector<Command*>& commands,
-                                  const SharedHandle<CheckIntegrityEntry>& entry,
-                                  DownloadEngine* e);
+  void
+  processCheckIntegrityEntry(std::vector<std::unique_ptr<Command>>& commands,
+                             std::unique_ptr<CheckIntegrityEntry> entry,
+                             DownloadEngine* e);
 
   // Initializes pieceStorage_ and segmentMan_.  We guarantee that
   // either both of pieceStorage_ and segmentMan_ are initialized or
@@ -395,44 +390,22 @@ public:
 
   bool downloadFinishedByFileLength();
 
-  void loadAndOpenFile(const SharedHandle<BtProgressInfoFile>& progressInfoFile);
+  void
+  loadAndOpenFile(const std::shared_ptr<BtProgressInfoFile>& progressInfoFile);
 
   void shouldCancelDownloadForSafety();
 
-  void adjustFilename(const SharedHandle<BtProgressInfoFile>& infoFile);
+  void adjustFilename(const std::shared_ptr<BtProgressInfoFile>& infoFile);
 
-  SharedHandle<DownloadResult> createDownloadResult() const;
+  std::shared_ptr<DownloadResult> createDownloadResult() const;
 
-  const SharedHandle<Option>& getOption() const
-  {
-    return option_;
-  }
+  const std::shared_ptr<Option>& getOption() const { return option_; }
 
   void reportDownloadFinished();
 
-  const std::vector<std::string>& getAcceptTypes() const
-  {
-    return acceptTypes_;
-  }
+  void setURISelector(std::unique_ptr<URISelector> uriSelector);
 
-  void addAcceptType(const std::string& type);
-
-  template<typename InputIterator>
-  void addAcceptType(InputIterator first, InputIterator last)
-  {
-    for(; first != last; ++first) {
-      if(std::find(acceptTypes_.begin(), acceptTypes_.end(), *first) ==
-	 acceptTypes_.end()) {
-	acceptTypes_.push_back(*first);
-      }
-    }
-  }
-
-  void removeAcceptType(const std::string& type);
-
-  void setURISelector(const SharedHandle<URISelector>& uriSelector);
-
-  const SharedHandle<URISelector>& getURISelector() const
+  const std::unique_ptr<URISelector>& getURISelector() const
   {
     return uriSelector_;
   }
@@ -447,17 +420,11 @@ public:
   void markInMemoryDownload();
 
   // Returns inMemoryDownload flag.
-  bool inMemoryDownload() const
-  {
-    return inMemoryDownload_;
-  }
+  bool inMemoryDownload() const { return inMemoryDownload_; }
 
-  void setTimeout(time_t timeout);
+  void setTimeout(std::chrono::seconds timeout);
 
-  time_t getTimeout() const
-  {
-    return timeout_;
-  }
+  const std::chrono::seconds& getTimeout() const { return timeout_; }
 
   // Returns true if current download speed exceeds
   // maxDownloadSpeedLimit_.  Always returns false if
@@ -469,35 +436,21 @@ public:
   // maxUploadSpeedLimit_ == 0. Otherwise returns false.
   bool doesUploadSpeedExceed();
 
-  int getMaxDownloadSpeedLimit() const
-  {
-    return maxDownloadSpeedLimit_;
-  }
+  int getMaxDownloadSpeedLimit() const { return maxDownloadSpeedLimit_; }
 
-  void setMaxDownloadSpeedLimit(int speed)
-  {
-    maxDownloadSpeedLimit_ = speed;
-  }
+  void setMaxDownloadSpeedLimit(int speed) { maxDownloadSpeedLimit_ = speed; }
 
-  int getMaxUploadSpeedLimit() const
-  {
-    return maxUploadSpeedLimit_;
-  }
+  int getMaxUploadSpeedLimit() const { return maxUploadSpeedLimit_; }
 
-  void setMaxUploadSpeedLimit(int speed)
-  {
-    maxUploadSpeedLimit_ = speed;
-  }
+  void setMaxUploadSpeedLimit(int speed) { maxUploadSpeedLimit_ = speed; }
 
-  void setLastErrorCode(error_code::Value code)
+  void setLastErrorCode(error_code::Value code, const char* message = "")
   {
     lastErrorCode_ = code;
+    lastErrorMessage_ = message;
   }
 
-  error_code::Value getLastErrorCode() const
-  {
-    return lastErrorCode_;
-  }
+  error_code::Value getLastErrorCode() const { return lastErrorCode_; }
 
   void saveControlFile() const;
 
@@ -507,70 +460,64 @@ public:
 
   void disableSaveControlFile() { saveControlFile_ = false; }
 
-  template<typename InputIterator>
+  template <typename InputIterator>
   void followedBy(InputIterator groupFirst, InputIterator groupLast)
   {
     followedByGIDs_.clear();
-    for(; groupFirst != groupLast; ++groupFirst) {
+    for (; groupFirst != groupLast; ++groupFirst) {
       followedByGIDs_.push_back((*groupFirst)->getGID());
     }
   }
 
-  const std::vector<a2_gid_t>& followedBy() const
-  {
-    return followedByGIDs_;
-  }
+  const std::vector<a2_gid_t>& followedBy() const { return followedByGIDs_; }
 
-  void belongsTo(a2_gid_t gid)
-  {
-    belongsToGID_ = gid;
-  }
+  void following(a2_gid_t gid) { followingGID_ = gid; }
 
-  a2_gid_t belongsTo() const
-  {
-    return belongsToGID_;
-  }
+  a2_gid_t following() const { return followingGID_; }
+
+  void belongsTo(a2_gid_t gid) { belongsToGID_ = gid; }
+
+  a2_gid_t belongsTo() const { return belongsToGID_; }
 
   void setRequestGroupMan(RequestGroupMan* requestGroupMan)
   {
     requestGroupMan_ = requestGroupMan;
   }
 
-  int getResumeFailureCount() const
-  {
-    return resumeFailureCount_;
-  }
+  RequestGroupMan* getRequestGroupMan() { return requestGroupMan_; }
 
-  void increaseResumeFailureCount()
-  {
-    ++resumeFailureCount_;
-  }
+  int getResumeFailureCount() const { return resumeFailureCount_; }
+
+  void increaseResumeFailureCount() { ++resumeFailureCount_; }
 
   bool p2pInvolved() const;
 
-  void setMetadataInfo(const SharedHandle<MetadataInfo>& info)
+  void setMetadataInfo(const std::shared_ptr<MetadataInfo>& info)
   {
     metadataInfo_ = info;
   }
 
-  const SharedHandle<MetadataInfo>& getMetadataInfo() const
+  const std::shared_ptr<MetadataInfo>& getMetadataInfo() const
   {
     return metadataInfo_;
   }
 
-  int getState() const
+  int getState() const { return state_; }
+
+  void setState(int state) { state_ = state; }
+
+  bool isSeedOnlyEnabled() { return seedOnly_; }
+
+  void enableSeedOnly();
+
+  // Returns true if this download is now seeding.
+  bool isSeeder() const;
+
+  void setPendingOption(std::shared_ptr<Option> option);
+  const std::shared_ptr<Option>& getPendingOption() const
   {
-    return state_;
+    return pendingOption_;
   }
-
-  void setState(int state)
-  {
-    state_ = state;
-  }
-
-  static void resetGIDCounter() { gidCounter_ = 0; }
-
-  static a2_gid_t newGID();
 };
 
 } // namespace aria2
