@@ -32,26 +32,22 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_REQUEST_GROUP_CRITERIA_H_
-#define _D_REQUEST_GROUP_CRITERIA_H_
+#ifndef D_REQUEST_GROUP_CRITERIA_H
+#define D_REQUEST_GROUP_CRITERIA_H
 
 #include "common.h"
-#include "SharedHandle.h"
 
 namespace aria2 {
 
 class RequestGroup;
 
-class RequestGroupCriteria
-{
+class RequestGroupCriteria {
 public:
   virtual ~RequestGroupCriteria() {}
 
   virtual bool match(const RequestGroup* requestGroup) const = 0;
 };
 
-typedef SharedHandle<RequestGroupCriteria> RequestGroupCriteriaHandle;
-
 } // namespace aria2
 
-#endif // _D_REQUEST_GROUP_CRITERIA_H_
+#endif // D_REQUEST_GROUP_CRITERIA_H
