@@ -38,11 +38,11 @@
 #include "EventPoll.h"
 
 #include <deque>
-#include <set>
+#include <map>
 
 #include "a2functional.h"
 #ifdef ENABLE_ASYNC_DNS
-# include "AsyncNameResolver.h"
+#include "AsyncNameResolver.h"
 #endif // ENABLE_ASYNC_DNS
 
 namespace aria2 {
@@ -53,53 +53,43 @@ private:
   private:
     Command* command_;
     int events_;
+
   public:
     CommandEvent(Command* command, int events);
 
-    Command* getCommand() const
-    {
-      return command_;
-    }
+    Command* getCommand() const { return command_; }
 
-    void addEvents(int events)
-    {
-      events_ |= events;
-    }
+    void addEvents(int events) { events_ |= events; }
 
-    void removeEvents(int events)
-    {
-      events_ &= (~events);
-    }
+    void removeEvents(int events) { events_ &= (~events); }
 
-    bool eventsEmpty() const
-    {
-      return events_ == 0;
-    }
+    bool eventsEmpty() const { return events_ == 0; }
 
     bool operator==(const CommandEvent& commandEvent) const
     {
       return command_ == commandEvent.command_;
     }
 
-    int getEvents() const
-    {
-      return events_;
-    }
+    int getEvents() const { return events_; }
 
     void processEvents(int events);
   };
 
-  friend int accumulateEvent
-  (int events, const SelectEventPoll::CommandEvent& event);
+  friend int accumulateEvent(int events,
+                             const SelectEventPoll::CommandEvent& event);
 
   class SocketEntry {
   private:
     sock_t socket_;
 
     std::deque<CommandEvent> commandEvents_;
+
   public:
     SocketEntry(sock_t socket);
 
+    SocketEntry(const SocketEntry&) = delete;
+    SocketEntry(SocketEntry&&) = default;
+
     bool operator==(const SocketEntry& entry) const
     {
       return socket_ == entry.socket_;
@@ -116,15 +106,9 @@ private:
 
     int getEvents();
 
-    sock_t getSocket() const
-    {
-      return socket_;
-    }
+    sock_t getSocket() const { return socket_; }
 
-    bool eventEmpty() const
-    {
-      return commandEvents_.empty();
-    }
+    bool eventEmpty() const { return commandEvents_.empty(); }
 
     void processEvents(int events);
   };
@@ -133,25 +117,29 @@ private:
 
   class AsyncNameResolverEntry {
   private:
-    SharedHandle<AsyncNameResolver> nameResolver_;
+    std::shared_ptr<AsyncNameResolver> nameResolver_;
 
     Command* command_;
 
   public:
-    AsyncNameResolverEntry(const SharedHandle<AsyncNameResolver>& nameResolver,
-                           Command* command);
+    AsyncNameResolverEntry(
+        const std::shared_ptr<AsyncNameResolver>& nameResolver,
+        Command* command);
+
+    AsyncNameResolverEntry(const AsyncNameResolverEntry&) = delete;
+    AsyncNameResolverEntry(AsyncNameResolverEntry&&) = default;
 
     bool operator==(const AsyncNameResolverEntry& entry)
     {
       return *nameResolver_ == *entry.nameResolver_ &&
-        command_ == entry.command_;
+             command_ == entry.command_;
     }
 
     bool operator<(const AsyncNameResolverEntry& entry)
     {
       return nameResolver_.get() < entry.nameResolver_.get() ||
-        (nameResolver_.get() == entry.nameResolver_.get() &&
-         command_ < entry.command_);
+             (nameResolver_.get() == entry.nameResolver_.get() &&
+              command_ < entry.command_);
     }
 
     int getFds(fd_set* rfdsPtr, fd_set* wfdsPtr);
@@ -164,13 +152,11 @@ private:
   fd_set wfdset_;
   sock_t fdmax_;
 
-  typedef std::set<SharedHandle<SocketEntry>,
-                   DerefLess<SharedHandle<SocketEntry> > > SocketEntrySet;
+  typedef std::map<sock_t, SocketEntry> SocketEntrySet;
   SocketEntrySet socketEntries_;
 #ifdef ENABLE_ASYNC_DNS
-  typedef std::set<SharedHandle<AsyncNameResolverEntry>,
-                   DerefLess<SharedHandle<AsyncNameResolverEntry> > >
-  AsyncNameResolverEntrySet;
+  typedef std::map<std::pair<AsyncNameResolver*, Command*>,
+                   AsyncNameResolverEntry> AsyncNameResolverEntrySet;
   AsyncNameResolverEntrySet nameResolverEntries_;
 #endif // ENABLE_ASYNC_DNS
 
@@ -181,24 +167,27 @@ private:
 #endif // __MINGW32__
 
   void updateFdSet();
+
 public:
   SelectEventPoll();
 
   virtual ~SelectEventPoll();
 
-  virtual void poll(const struct timeval& tv);
+  virtual void poll(const struct timeval& tv) CXX11_OVERRIDE;
 
-  virtual bool addEvents(sock_t socket,
-                         Command* command, EventPoll::EventType events);
+  virtual bool addEvents(sock_t socket, Command* command,
+                         EventPoll::EventType events) CXX11_OVERRIDE;
 
-  virtual bool deleteEvents(sock_t socket,
-                            Command* command, EventPoll::EventType events);
+  virtual bool deleteEvents(sock_t socket, Command* command,
+                            EventPoll::EventType events) CXX11_OVERRIDE;
 #ifdef ENABLE_ASYNC_DNS
 
-  virtual bool addNameResolver(const SharedHandle<AsyncNameResolver>& resolver,
-                               Command* command);
-  virtual bool deleteNameResolver
-  (const SharedHandle<AsyncNameResolver>& resolver, Command* command);
+  virtual bool
+  addNameResolver(const std::shared_ptr<AsyncNameResolver>& resolver,
+                  Command* command) CXX11_OVERRIDE;
+  virtual bool
+  deleteNameResolver(const std::shared_ptr<AsyncNameResolver>& resolver,
+                     Command* command) CXX11_OVERRIDE;
 #endif // ENABLE_ASYNC_DNS
 };
 
