@@ -32,34 +32,33 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_SINGLETON_HOLDER_H_
-#define _D_SINGLETON_HOLDER_H_
+#ifndef D_SINGLETON_HOLDER_H
+#define D_SINGLETON_HOLDER_H
+
+#include "common.h"
+
+#include <memory>
 
 namespace aria2 {
 
-template<typename T>
-class SingletonHolder {
+template <typename T> class SingletonHolder {
 private:
-  static T instance_;
+  static std::unique_ptr<T> instance_;
 
   SingletonHolder() {}
+
 public:
   ~SingletonHolder() {}
 
-  static T& instance()
-  {
-    return instance_;
-  }
+  static std::unique_ptr<T>& instance() { return instance_; }
 
-  static void instance(T& instance)
-  {
-    instance_ = instance;
-  }
+  static void instance(std::unique_ptr<T> ptr) { instance_ = std::move(ptr); }
+
+  static void clear() { instance_.reset(); }
 };
 
-template<typename T>
-T SingletonHolder<T>::instance_;
+template <typename T> std::unique_ptr<T> SingletonHolder<T>::instance_;
 
 } // namespace aria2
 
-#endif // _D_SINGLETON_HOLDER_H_
+#endif // D_SINGLETON_HOLDER_H
