@@ -38,9 +38,10 @@
 #define TEXT_DIR                                                        \
   _(" -d, --dir=DIR                The directory to store the downloaded file.")
 #define TEXT_OUT                                                        \
-  _(" -o, --out=FILE               The file name of the downloaded file. When \n" \
-    "                              the -Z option is used, this option will be \n" \
-    "                              ignored.")
+  _(" -o, --out=FILE               The file name of the downloaded file. It is\n" \
+    "                              always relative to the directory given in -d\n" \
+    "                              option. When the -Z option is used, this option\n" \
+    "                              will be ignored.")
 #define TEXT_LOG                                                        \
   _(" -l, --log=LOG                The file name of the log file. If '-' is\n" \
     "                              specified, log is written to stdout.")
@@ -538,9 +539,13 @@
 #define TEXT_EVENT_POLL                                                 \
   _(" --event-poll=POLL            Specify the method for polling events.")
 #define TEXT_BT_EXTERNAL_IP                                             \
-  _(" --bt-external-ip=IPADDRESS   Specify the external IP address to report to a\n" \
-    "                              BitTorrent tracker. Although this function is\n" \
-    "                              named 'external', it can accept any kind of IP\n" \
+  _(" --bt-external-ip=IPADDRESS   Specify the external IP address to use in\n" \
+    "                              BitTorrent download and DHT. It may be sent to\n" \
+    "                              BitTorrent tracker. For DHT, this option should\n" \
+    "                              be set to report that local node is downloading\n" \
+    "                              a particular torrent. This is critical to use\n" \
+    "                              DHT in a private network. Although this function\n" \
+    "                              is named 'external', it can accept any kind of IP\n" \
     "                              addresses.")
 #define TEXT_HTTP_AUTH_CHALLENGE                                        \
   _(" --http-auth-challenge[=true|false] Send HTTP authorization header only when it\n" \
@@ -882,7 +887,8 @@
     "                              download speed, percentage of progress and\n" \
     "                              path/URI. The percentage of progress and\n" \
     "                              path/URI are printed for each requested file in\n" \
-    "                              each row.")
+    "                              each row.\n" \
+    "                              If OPT is 'hide', \"Download Results\" is hidden.")
 #define TEXT_HASH_CHECK_ONLY                    \
   _(" --hash-check-only[=true|false] If true is given, after hash check using\n" \
     "                              --check-integrity option, abort download whether\n" \
@@ -1084,5 +1090,8 @@
     "                              size of those files. If file size is strictly\n" \
     "                              greater than the size specified in this option,\n" \
     "                              mmap will be disabled.")
+#define TEXT_STDERR \
+  _(" --stderr[=true|false]        Redirect all console output that would be\n" \
+    "                              otherwise printed in stdout to stderr.")
 
 // clang-format on
